@@ -1,29 +1,10 @@
-                        /*** /
+// This file is part of Golly.
+// See docs/License.html for the copyright notice.
 
-This file is part of Golly, a Game of Life Simulator.
-Copyright (C) 2013 Andrew Trevorrow and Tomas Rokicki.
-
-This program is free software; you can redistribute it and/or
-modify it under the terms of the GNU General Public License
-as published by the Free Software Foundation; either version 2
-of the License, or (at your option) any later version.
-
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
-
-You should have received a copy of the GNU General Public License
-along with this program; if not, write to the Free Software
-Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
-
- Web site:  http://sourceforge.net/projects/golly
- Authors:   rokicki@gmail.com  andrew@trevorrow.com
-
-                        / ***/
 #include "qlifealgo.h"
 #include "hlifealgo.h"
 #include "generationsalgo.h"
+#include "ltlalgo.h"
 #include "jvnalgo.h"
 #include "ruleloaderalgo.h"
 #include "readpattern.h"
@@ -36,23 +17,20 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 #include <cstdio>
 #include <string.h>
 #include <cstdlib>
-#ifdef TIMING
-#include <sys/time.h>
-#endif
 
 using namespace std ;
 
-#ifdef TIMING
 double start ;
+int maxtime = 0 ;
 double timestamp() {
-   struct timeval tv ;
-   gettimeofday(&tv, 0) ;
-   double now = tv.tv_sec + 0.000001 * tv.tv_usec ;
+   double now = gollySecondCount() ;
    double r = now - start ;
-   start = now ;
+   if (start == 0)
+      start = now ;
+   else if (maxtime && r > maxtime)
+      exit(0) ;   
    return r ;
 }
-#endif
 
 viewport viewport(1000, 1000) ;
 lifealgo *imp = 0 ;
@@ -79,6 +57,7 @@ nullrender renderer ;
 char* user_rules = (char *)"";              // can be changed by -s or --search
 char* supplied_rules = (char *)"Rules/";
 
+int benchmark ; // show timing?
 /*
  *   This lifeerrors is used to check rendering during a progress dialog.
  */
@@ -87,7 +66,14 @@ public:
    progerrors() {}
    virtual void fatal(const char *s) { cout << "Fatal error: " << s << endl ; exit(10) ; }
    virtual void warning(const char *s) { cout << "Warning: " << s << endl ; }
-   virtual void status(const char *s) { cout << s << endl ; }
+   virtual void status(const char *s) { 
+      if (benchmark)
+         cout << timestamp() << " " << s << endl ;
+      else {
+         timestamp() ;
+         cout << s << endl ;
+      }
+   }
    virtual void beginprogress(const char *s) { abortprogress(0, s) ; }
    virtual bool abortprogress(double, const char *) ;
    virtual void endprogress() { abortprogress(1, "") ; }
@@ -110,11 +96,12 @@ public:
    virtual void fatal(const char *s) { cout << "Fatal error: " << s << endl ; exit(10) ; }
    virtual void warning(const char *s) { cout << "Warning: " << s << endl ; }
    virtual void status(const char *s) {
-#ifdef TIMING
-      cout << timestamp() << " " << s << endl ;
-#else
-      cout << s << endl ;
-#endif
+      if (benchmark)
+         cout << timestamp() << " " << s << endl ;
+      else {
+         timestamp() ;
+         cout << s << endl ;
+      }
    }
    virtual void beginprogress(const char *) {}
    virtual bool abortprogress(double, const char *) { return 0 ; }
@@ -134,7 +121,8 @@ struct options {
 } ;
 bigint maxgen = -1, inc = 0 ;
 int maxmem = 256 ;
-int hyper, render, autofit, quiet, popcount, progress ;
+int hyperxxx ;   // renamed hyper to avoid conflict with windows.h
+int render, autofit, quiet, popcount, progress ;
 int hashlife ;
 char *algoName = 0 ;
 int verbose ;
@@ -150,7 +138,9 @@ options options[] = {
   { "-m", "--generation", "How far to run", 'I', &maxgen },
   { "-i", "--stepsize", "Step size", 'I', &inc },
   { "-M", "--maxmemory", "Max memory to use in megabytes", 'i', &maxmem },
-  { "-2", "--exponential", "Use exponentially increasing steps", 'b', &hyper },
+  { "-T", "--maxtime", "Max duration", 'i', &maxtime },
+  { "-b", "--benchmark", "Show timestamps", 'b', &benchmark },
+  { "-2", "--exponential", "Use exponentially increasing steps", 'b', &hyperxxx },
   { "-q", "--quiet", "Don't show population; twice, don't show anything", 'b', &quiet },
   { "-r", "--rule", "Life rule to use", 's', &liferule },
   { "-s", "--search", "Search directory for .rule files", 's', &user_rules },
@@ -334,7 +324,7 @@ struct loadcmd : public cmdbase {
 struct stepcmd : public cmdbase {
    stepcmd() : cmdbase("step", "b") {}
    virtual void doit() {
-      if (imp->gridwd > 0 || imp->gridht > 0) {
+      if (imp->unbounded && (imp->gridwd > 0 || imp->gridht > 0)) {
          // bounded grid, so must step by 1
          imp->setIncrement(1) ;
          if (!imp->CreateBorderCells()) exit(10) ;
@@ -450,8 +440,10 @@ lifealgo *createUniverse() {
        algoName = (char *)"RuleLoader" ;
    }
    staticAlgoInfo *ai = staticAlgoInfo::byName(algoName) ;
-   if (ai == 0)
+   if (ai == 0) {
+      cout << algoName << endl ; //!!!
       lifefatal("No such algorithm") ;
+   }
    lifealgo *imp = (ai->creator)() ;
    if (imp == 0)
       lifefatal("Could not create universe") ;
@@ -520,11 +512,16 @@ void runtestscript(const char *testscript) {
 }
 
 int main(int argc, char *argv[]) {
-   cout << "This is bgolly " STRINGIFY(VERSION) " Copyright 2016 The Golly Gang."
-        << endl << flush ;
+   cout << "This is bgolly " STRINGIFY(VERSION) " Copyright 2005-2018 The Golly Gang."
+        << endl ;
+   cout << "-" ;
+   for (int i=0; i<argc; i++)
+      cout << " " << argv[i] ;
+   cout << endl << flush ;
    qlifealgo::doInitializeAlgoInfo(staticAlgoInfo::tick()) ;
    hlifealgo::doInitializeAlgoInfo(staticAlgoInfo::tick()) ;
    generationsalgo::doInitializeAlgoInfo(staticAlgoInfo::tick()) ;
+   ltlalgo::doInitializeAlgoInfo(staticAlgoInfo::tick()) ;
    jvnalgo::doInitializeAlgoInfo(staticAlgoInfo::tick()) ;
    ruleloaderalgo::doInitializeAlgoInfo(staticAlgoInfo::tick()) ;
    while (argc > 1 && argv[1][0] == '-') {
@@ -589,7 +586,7 @@ case 's':
       if (strlen(outfilename) > 200)
          lifefatal("Output filename too long") ;
    }
-   if (timeline && hyper)
+   if (timeline && hyperxxx)
       lifefatal("Cannot use both timeline and exponentially increasing steps") ;
    imp = createUniverse() ;
    if (progress)
@@ -600,9 +597,7 @@ case 's':
       hlifealgo::setVerbose(1) ;
    }
    imp->setMaxMemory(maxmem) ;
-#ifdef TIMING
    timestamp() ;
-#endif
    if (testscript) {
       if (argc > 1) {
          filename = argv[1] ;
@@ -618,9 +613,9 @@ case 's':
       err = imp->setrule(liferule) ;
       if (err) lifefatal(err) ;
    }
-   bool boundedgrid = (imp->gridwd > 0 || imp->gridht > 0) ;
+   bool boundedgrid = imp->unbounded && (imp->gridwd > 0 || imp->gridht > 0) ;
    if (boundedgrid) {
-      hyper = 0 ;
+      hyperxxx = 0 ;
       inc = 1 ;     // only step by 1
    }
    if (inc != 0)
@@ -636,11 +631,21 @@ case 's':
    }
    int fc = 0 ;
    for (;;) {
+   if (benchmark)
+      cout << timestamp() << " " ;
+   else
+      timestamp() ;
       if (quiet < 2) {
          cout << imp->getGeneration().tostring() ;
-         if (!quiet)
-            cout << ": " << imp->getPopulation().tostring() << endl ;
-         else
+         if (!quiet) {
+            const char *s = imp->getPopulation().tostring() ;
+            if (benchmark) {
+               cout << endl ;
+               cout << timestamp() << " pop " << s << endl ;
+            } else {
+               cout << ": " << s << endl ;
+            }
+         } else
             cout << endl ;
       }
       if (popcount)
@@ -651,7 +656,7 @@ case 's':
         imp->draw(viewport, renderer) ;
       if (maxgen >= 0 && imp->getGeneration() >= maxgen)
          break ;
-      if (!hyper && maxgen > 0 && inc == 0) {
+      if (!hyperxxx && maxgen > 0 && inc == 0) {
          bigint diff = maxgen ;
          diff -= imp->getGeneration() ;
          int bs = diff.lowbitset() ;
@@ -667,7 +672,7 @@ case 's':
          writepat(fc++) ;
       if (timeline && imp->getframecount() + 2 > MAX_FRAME_COUNT)
          imp->pruneframes() ;
-      if (hyper)
+      if (hyperxxx)
          imp->setIncrement(imp->getGeneration()) ;
    }
    if (maxgen >= 0 && outfilename != 0)
