@@ -1,26 +1,6 @@
-                        /*** /
+// This file is part of Golly.
+// See docs/License.html for the copyright notice.
 
-This file is part of Golly, a Game of Life Simulator.
-Copyright (C) 2013 Andrew Trevorrow and Tomas Rokicki.
-
-This program is free software; you can redistribute it and/or
-modify it under the terms of the GNU General Public License
-as published by the Free Software Foundation; either version 2
-of the License, or (at your option) any later version.
-
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
-
-You should have received a copy of the GNU General Public License
-along with this program; if not, write to the Free Software
-Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
-
- Web site:  http://sourceforge.net/projects/golly
- Authors:   rokicki@gmail.com  andrew@trevorrow.com
-
-                        / ***/
 /*
  *   Inspired by Alan Hensel's Life applet and also by xlife.  Tries to
  *   improve the cache, TLB, and branching behavior for modern CPUs.
@@ -918,7 +898,7 @@ int qlifealgo::nextcell(int x, int y, supertile *n, int lev) {
  */
 G_INT64 qlifealgo::find_set_bits(supertile *p, int lev, int gm1) {
    G_INT64 pop = 0 ;
-   int i, j, k, b ;
+   int i, j, b ;
    if (lev == 0) {
       tile *pp = (tile *)p ;
       b = 8 + gm1 * 12 ;
@@ -928,10 +908,14 @@ G_INT64 qlifealgo::find_set_bits(supertile *p, int lev, int gm1) {
          for (i=0; i<4; i++) {
             if (pp->b[i] != emptybrick) {
                for (j=0; j<8; j++) {
-                  k = pp->b[i]->d[j+gm1*8] ;
+#ifdef FASTPOPCOUNT
+                  pop += FASTPOPCOUNT(pp->b[i]->d[j+gm1*8]) ;
+#else
+                  int k = pp->b[i]->d[j+gm1*8] ;
                   if (k)
                     pop += bc[k & 255] + bc[(k >> 8) & 255] +
                       bc[(k >> 16) & 255] + bc[(k >> 24) & 255] ;
+#endif
                }
             }
          }
@@ -943,7 +927,7 @@ G_INT64 qlifealgo::find_set_bits(supertile *p, int lev, int gm1) {
             if (p->d[i] != nullroots[lev-1])
                pop += find_set_bits(p->d[i], lev-1, gm1) ;
          if (pop < 500000000) {
-            p->pop[gm1] = (long)pop ;
+            p->pop[gm1] = (int)pop ;
             p->flags &= ~(0x20000000 << gm1) ;
          } else {
             p->pop[gm1] = 0xfffffff ; // placeholder; *some* bits are set
