@@ -1,26 +1,6 @@
-                        /*** /
+// This file is part of Golly.
+// See docs/License.html for the copyright notice.
 
-This file is part of Golly, a Game of Life Simulator.
-Copyright (C) 2013 Andrew Trevorrow and Tomas Rokicki.
-
-This program is free software; you can redistribute it and/or
-modify it under the terms of the GNU General Public License
-as published by the Free Software Foundation; either version 2
-of the License, or (at your option) any later version.
-
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
-
-You should have received a copy of the GNU General Public License
-along with this program; if not, write to the Free Software
-Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
-
- Web site:  http://sourceforge.net/projects/golly
- Authors:   rokicki@gmail.com  andrew@trevorrow.com
-
-                        / ***/
 #include "readpattern.h"
 #include "lifealgo.h"
 #include "liferules.h"     // for MAXRULESIZE
@@ -223,9 +203,11 @@ const char *readrle(lifealgo &imp, char *line) {
       } else if (line[0] == 'x' && (line[1] <= ' ' || line[1] == '=')) {
          // extract wd and ht
          p = line;
-         while (*p && *p != '=') p++; p++;
+         while (*p && *p != '=') p++;
+         p++;
          sscanf(p, "%d", &wd);
-         while (*p && *p != '=') p++; p++;
+         while (*p && *p != '=') p++;
+         p++;
          sscanf(p, "%d", &ht);
 
          while (*p && *p != 'r') p++;
@@ -251,9 +233,18 @@ const char *readrle(lifealgo &imp, char *line) {
 
          // imp.setrule() has set imp.gridwd and imp.gridht
          if (!sawpos && (imp.gridwd > 0 || imp.gridht > 0)) {
-            // position pattern at top left corner of bounded grid
-            xoff = -int(imp.gridwd / 2);
-            yoff = -int(imp.gridht / 2);
+            if (wd > 0 && (wd <= (int)imp.gridwd || imp.gridwd == 0) &&
+                ht > 0 && (ht <= (int)imp.gridht || imp.gridht == 0)) {
+                // pattern size is known and fits within the bounded grid
+                // so position pattern in the middle of the grid
+                xoff = -int(wd / 2);
+                yoff = -int(ht / 2);
+            } else {
+                // position pattern at top left corner of bounded grid
+                // to try and ensure the entire pattern will fit
+                xoff = -int(imp.gridwd / 2);
+                yoff = -int(imp.gridht / 2);
+            }
          }
 
          if (getedges) {
@@ -263,7 +254,8 @@ const char *readrle(lifealgo &imp, char *line) {
             right = xoff + wd - 1;
          }
       } else {
-         n = 0 ;
+         int gwd = (int)imp.gridwd;
+         int ght = (int)imp.gridht;
          for (p=line; *p; p++) {
             char c = *p ;
             if ('0' <= c && c <= '9') {
@@ -298,9 +290,15 @@ const char *readrle(lifealgo &imp, char *line) {
                         p-- ;
                      }
                   }
-                  while (n-- > 0) {
-                     if (imp.setcell(xoff + x++, yoff + y, state) < 0)
-                        return "Cell state out of range for this algorithm" ;
+                  // write run of cells to grid checking cells are within any bounded grid
+                  if (ght == 0 || y < ght) {
+                     while (n-- > 0) {
+                        if (gwd == 0 || x < gwd) {  
+                           if (imp.setcell(xoff + x, yoff + y, state) < 0)
+                              return "Cell state out of range for this algorithm" ;
+                        }
+                        x++;
+                     }
                   }
                }
                n = 0 ;
@@ -417,6 +415,10 @@ const char *readmcell(lifealgo &imp, char *line) {
    const char *errmsg;
    bool sawrule = false;            // saw explicit rule?
    bool extendedHL = false;         // special-case rule translation for extended HistoricalLife rules
+   bool useltl = false;             // using a Larger than Life rule?
+   char ltlrule[MAXRULESIZE];       // the Larger than Life rule (without any suffix)
+   int defwd = 0, defht = 0;        // default grid size for Larger than Life
+   int Lcount = 0;                  // number of #L lines seen
 
    while (getline(line, LINESIZE)) {
       if (line[0] == '#') {
@@ -429,6 +431,24 @@ const char *readmcell(lifealgo &imp, char *line) {
                sawrule = true;
             }
 
+            Lcount++;
+            if (Lcount == 1 && useltl) {
+                // call setrule again with correct width and height so we don't need
+                // yet another setrule call at the end
+                char rule[MAXRULESIZE];
+                if (wd == 0 && ht == 0) {
+                    // no #BOARD line was seen so use default size saved earlier
+                    wd = defwd;
+                    ht = defht;
+                }
+                if (wrapped)
+                    sprintf(rule, "%s:T%d,%d", ltlrule, wd, ht);
+                else
+                    sprintf(rule, "%s:P%d,%d", ltlrule, wd, ht);
+                errmsg = imp.setrule(rule);
+                if (errmsg) return errmsg;
+            }
+
             int n = 0;
             for (p=line+3; *p; p++) {
                char c = *p;
@@ -461,14 +481,23 @@ const char *readmcell(lifealgo &imp, char *line) {
                         return "Illegal multi-char state";
                      }
                      while (n-- > 0) {
-                        if (imp.setcell(x++, y, state) < 0)
-                           return "Cell state out of range";
+                        if (imp.setcell(x, y, state) < 0) {
+                           // instead of returning an error message like "Cell state out of range"
+                           // it's nicer to convert the state to 1 (this is what MCell seems to do
+                           // for patterns like Bug1.mcl in its LtL collection)
+                           imp.setcell(x, y, 1);
+                        }
+                        x++;
                      }
                   }
                   n = 0;
                }
             }
 
+         // look for Larger than Life
+         } else if (strncmp(line, "#GAME Larger than Life", 22) == 0) {
+            useltl = true;
+
          // look for bounded universe
          } else if (strncmp(line, "#BOARD ", 7) == 0) {
             sscanf(line + 7, "%dx%d", &wd, &ht);
@@ -510,6 +539,13 @@ const char *readmcell(lifealgo &imp, char *line) {
                *p = 0;
                errmsg = imp.setrule(ruleptr);
                if (errmsg) return errmsg;
+               if (useltl) {
+                  // save suffix-less rule string for later use when we see first #L line
+                  sprintf(ltlrule, "%s", ruleptr);
+                  // also save default grid size in case there is no #BOARD line
+                  defwd = imp.gridwd;
+                  defht = imp.gridht;
+               }
                sawrule = true;
             }
          }
@@ -517,17 +553,21 @@ const char *readmcell(lifealgo &imp, char *line) {
    }
    
    if (wd > 0 || ht > 0) {
-      // grid is bounded, so append suitable suffix to rule
-      char rule[MAXRULESIZE];
-      if (wrapped)
-         sprintf(rule, "%s:T%d,%d", imp.getrule(), wd, ht);
-      else
-         sprintf(rule, "%s:P%d,%d", imp.getrule(), wd, ht);
-      errmsg = imp.setrule(rule);
-      if (errmsg) {
-         // should never happen
-         lifewarning("Bug in readmcell code!");
-         return errmsg;
+      if (useltl) {
+         // setrule has already been called above
+      } else {
+         // grid is bounded, so append suitable suffix to rule
+         char rule[MAXRULESIZE];
+         if (wrapped)
+            sprintf(rule, "%s:T%d,%d", imp.getrule(), wd, ht);
+         else
+            sprintf(rule, "%s:P%d,%d", imp.getrule(), wd, ht);
+         errmsg = imp.setrule(rule);
+         if (errmsg) {
+            // should never happen
+            lifewarning("Bug in readmcell code!");
+            return errmsg;
+         }
       }
       // shift pattern to middle of bounded grid
       imp.endofpattern();
@@ -913,7 +953,7 @@ const char *readcomments(const char *filename, char **commptr)
       }
 
    } else {
-      // no comments in text pattern file???
+      // no comments in text pattern file
    }
 
    lifeendprogress();
