@@ -1,26 +1,5 @@
-/*** /
-
- This file is part of Golly, a Game of Life Simulator.
- Copyright (C) 2013 Andrew Trevorrow and Tomas Rokicki.
-
- This program is free software; you can redistribute it and/or
- modify it under the terms of the GNU General Public License
- as published by the Free Software Foundation; either version 2
- of the License, or (at your option) any later version.
-
- This program is distributed in the hope that it will be useful,
- but WITHOUT ANY WARRANTY; without even the implied warranty of
- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- GNU General Public License for more details.
-
- You should have received a copy of the GNU General Public License
- along with this program; if not, write to the Free Software
- Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
-
- Web site:  http://sourceforge.net/projects/golly
- Authors:   rokicki@gmail.com  andrew@trevorrow.com
-
- / ***/
+// This file is part of Golly.
+// See docs/License.html for the copyright notice.
 
 #include <jni.h>        // for calling Java from C++ and vice versa
 #include <GLES/gl.h>    // for OpenGL ES 1.x calls
@@ -35,7 +14,7 @@
 #include "prefs.h"      // for GetPrefs, SavePrefs, userdir, etc
 #include "layer.h"      // for AddLayer, ResizeLayers, currlayer
 #include "control.h"    // for SetMinimumStepExponent
-#include "file.h"       // for NewPattern
+#include "file.h"       // for NewPattern, GetURL, ProcessDownload
 #include "render.h"     // for DrawPattern
 #include "view.h"       // for widescreen, fullscreen, TouchBegan, etc
 #include "status.h"     // for UpdateStatusLines, ClearMessage, etc
@@ -340,7 +319,7 @@ JNIEXPORT void JNICALL Java_net_sf_golly_BaseApp_nativeCreate(JNIEnv* env, jobje
     if (baseapp != NULL) env->DeleteGlobalRef(baseapp);
     baseapp = env->NewGlobalRef(obj);
 
-    SetMessage("This is Golly 1.2 for Android.  Copyright 2016 The Golly Gang.");
+    SetMessage("This is Golly 1.2 for Android.  Copyright 2005-2018 The Golly Gang.");
     if (highdensity) {
         MAX_MAG = 6;            // maximum cell size = 64x64
     } else {
@@ -830,9 +809,11 @@ JNIEXPORT int JNICALL Java_net_sf_golly_MainActivity_nativeGetMode(JNIEnv* env)
         case pickmode:   return 1;
         case selectmode: return 2;
         case movemode:   return 3;
+        default:
+            Warning("Bug detected in nativeGetMode!");
+            return 0;
     }
-    Warning("Bug detected in nativeGetMode!");
-    return 0;
+
 }
 
 // -----------------------------------------------------------------------------
@@ -845,8 +826,11 @@ JNIEXPORT void JNICALL Java_net_sf_golly_MainActivity_nativeSetMode(JNIEnv* env,
         case 1: currlayer->touchmode = pickmode;   return;
         case 2: currlayer->touchmode = selectmode; return;
         case 3: currlayer->touchmode = movemode;   return;
+        default:
+            Warning("Bug detected in nativeSetMode!");
+            return;
     }
-    Warning("Bug detected in nativeSetMode!");
+
 }
 
 // -----------------------------------------------------------------------------
@@ -1632,7 +1616,7 @@ extern "C"
 JNIEXPORT void JNICALL Java_net_sf_golly_HelpActivity_nativeClassInit(JNIEnv* env, jclass klass)
 {
     // get IDs for Java methods in HelpActivity
-    help_DownloadFile = env->GetMethodID(klass, "DownloadFile", "(Ljava/lang/String;Ljava/lang/String;)Ljava/lang/String;");
+    help_DownloadFile = env->GetMethodID(klass, "DownloadFile", "(Ljava/lang/String;Ljava/lang/String;)V");
 }
 
 // -----------------------------------------------------------------------------
@@ -1706,15 +1690,26 @@ JNIEXPORT bool JNICALL Java_net_sf_golly_HelpActivity_nativeDownloadedFile(JNIEn
         // download file to downloaddir and open it
         path = downloaddir + path;
         std::string url = ConvertJString(env, jurl);
-        if (DownloadFile(url, path)) {
-            OpenFile(path.c_str());
-        }
+        AndroidDownloadFile(url, path);
+        // if the async download succeeds then nativeProcessDownload will call
+        // ProcessDownload which calls OpenFile
         return true;
     } else {
         return false;
     }
 }
 
+// -----------------------------------------------------------------------------
+
+extern "C"
+JNIEXPORT void JNICALL Java_net_sf_golly_HelpActivity_nativeProcessDownload(JNIEnv* env, jobject obj, jstring jfilepath)
+{
+    std::string filepath = ConvertJString(env, jfilepath);
+
+    // AndroidDownloadFile has successfully downloaded and created a file
+    ProcessDownload(filepath);
+}
+
 // =============================================================================
 
 // these native routines are used in StateActivity.java:
@@ -2439,6 +2434,7 @@ void ShowTextFile(const char* filepath)
 void ShowHelp(const char* filepath)
 {
     // switch to HelpActivity and display html file
+    // LOGI("ShowHelp: filepath=%s", filepath);
     bool attached;
     JNIEnv* env = getJNIenv(&attached);
     if (env) {
@@ -2622,10 +2618,9 @@ void AndroidCheckEvents()
 
 // -----------------------------------------------------------------------------
 
-bool AndroidDownloadFile(const std::string& url, const std::string& filepath)
+void AndroidDownloadFile(const std::string& url, const std::string& filepath)
 {
     // LOGI("AndroidDownloadFile: url=%s file=%s", url.c_str(), filepath.c_str());
-    std::string error = "env is null";
 
     // call DownloadFile method in HelpActivity
     bool attached;
@@ -2633,13 +2628,9 @@ bool AndroidDownloadFile(const std::string& url, const std::string& filepath)
     if (env) {
         jstring jurl = env->NewStringUTF(url.c_str());
         jstring jfilepath = env->NewStringUTF(filepath.c_str());
-        jstring jresult = (jstring) env->CallObjectMethod(helpobj, help_DownloadFile, jurl, jfilepath);
-        error = ConvertJString(env, jresult);
+        env->CallVoidMethod(helpobj, help_DownloadFile, jurl, jfilepath);
         env->DeleteLocalRef(jurl);
         env->DeleteLocalRef(jfilepath);
-        env->DeleteLocalRef(jresult);
     }
     if (attached) javavm->DetachCurrentThread();
-
-    return error.length() == 0;
 }
