@@ -1,26 +1,5 @@
-/*** /
-
- This file is part of Golly, a Game of Life Simulator.
- Copyright (C) 2013 Andrew Trevorrow and Tomas Rokicki.
-
- This program is free software; you can redistribute it and/or
- modify it under the terms of the GNU General Public License
- as published by the Free Software Foundation; either version 2
- of the License, or (at your option) any later version.
-
- This program is distributed in the hope that it will be useful,
- but WITHOUT ANY WARRANTY; without even the implied warranty of
- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- GNU General Public License for more details.
-
- You should have received a copy of the GNU General Public License
- along with this program; if not, write to the Free Software
- Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
-
- Web site:  http://sourceforge.net/projects/golly
- Authors:   rokicki@gmail.com  andrew@trevorrow.com
-
- / ***/
+// This file is part of Golly.
+// See docs/License.html for the copyright notice.
 
 #include "bigint.h"
 #include "lifealgo.h"
@@ -40,6 +19,7 @@
 #include "layer.h"
 
 #include <map>              // for std::map
+#include <algorithm>        // for std::replace
 
 // -----------------------------------------------------------------------------
 
@@ -410,7 +390,6 @@ void CurrentLayerChanged()
 
     mainptr->UpdateUserInterface(mainptr->IsActive());
     mainptr->UpdatePatternAndStatus();
-    bigview->UpdateScrollBars();
 }
 
 !!!*/
@@ -791,7 +770,7 @@ void NameLayerDialog()
 {
     wxString oldname = currlayer->currname;
     wxString newname;
-    if ( GetString(_("Name Layer"), _("Enter a new name for the current layer:"),
+    if ( GetString("Name Layer", "Enter a new name for the current layer:",
                    oldname, newname) &&
         !newname.IsEmpty() && oldname != newname ) {
 
@@ -1232,21 +1211,33 @@ static void ParseIcons(const std::string& rulename, linereader& reader, char* li
                 }
 
                 // copy data inside "..." to next string in xpmdata
-                int len = strlen(linebuf);
+                int len = (int)strlen(linebuf);
                 while (linebuf[len] != '"') len--;
                 len--;
                 if (xpmstrings > 0 && xpmstrings <= numcolors) {
                     // build colormap so we can validate chars in pixel data
                     std::string pixel;
-                    char ch1, ch2;
+                    char ch1, ch2, ch3;
+                    bool badline = false;
                     if (chars_per_pixel == 1) {
-                        sscanf(linebuf+1, "%c ", &ch1);
+                        badline = sscanf(linebuf+1, "%c%c", &ch1, &ch2) != 2 || ch2 != ' ';
                         pixel += ch1;
                     } else {
-                        sscanf(linebuf+1, "%c%c ", &ch1, &ch2);
+                        badline = sscanf(linebuf+1, "%c%c%c", &ch1, &ch2, &ch3) != 3 || ch3 != ' ';
                         pixel += ch1;
                         pixel += ch2;
                     }
+                    if (badline) {
+                        DeleteXPMData(xpmdata, maxstrings);
+                        char s[128];
+                        sprintf(s, "The XPM data string on line %d in ", *linenum);
+                        std::string msg(s);
+                        msg += rulename;
+                        msg += ".rule is incorrect.";
+                        Warning(msg.c_str());
+                        *eof = true;
+                        return;
+                    }
                     colormap[pixel] = xpmstrings;
                 } else if (xpmstrings > numcolors) {
                     // check length of string containing pixel data
