@@ -1,26 +1,5 @@
-/*** /
-
- This file is part of Golly, a Game of Life Simulator.
- Copyright (C) 2013 Andrew Trevorrow and Tomas Rokicki.
-
- This program is free software; you can redistribute it and/or
- modify it under the terms of the GNU General Public License
- as published by the Free Software Foundation; either version 2
- of the License, or (at your option) any later version.
-
- This program is distributed in the hope that it will be useful,
- but WITHOUT ANY WARRANTY; without even the implied warranty of
- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- GNU General Public License for more details.
-
- You should have received a copy of the GNU General Public License
- along with this program; if not, write to the Free Software
- Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
-
- Web site:  http://sourceforge.net/projects/golly
- Authors:   rokicki@gmail.com  andrew@trevorrow.com
-
- / ***/
+// This file is part of Golly.
+// See docs/License.html for the copyright notice.
 
 #include "bigint.h"
 #include "lifealgo.h"
@@ -31,6 +10,7 @@
 #include "layer.h"      // for currlayer
 #include "view.h"       // for nopattupdate, widescreen, PointInView, etc
 #include "status.h"
+#include <math.h>       // for fabs
 
 #ifdef ANDROID_GUI
     #include "jnicalls.h"   // for UpdateStatus, GetRuleName
@@ -38,7 +18,6 @@
 
 #ifdef WEB_GUI
     #include "webcalls.h"   // for UpdateStatus, GetRuleName
-    #include <math.h>       // for fabs
 #endif
 
 #ifdef IOS_GUI
@@ -224,7 +203,7 @@ char* Stringify(const bigint& b)
             *p++ = '-';
         }
         sprintf(p, "%.f", d);
-        int len = strlen(p);
+        int len = (int)strlen(p);
         int commas = ((len + 2) / 3) - 1;
         int dest = len + commas;
         int src = len;
