@@ -1,33 +1,12 @@
-/*** /
-
- This file is part of Golly, a Game of Life Simulator.
- Copyright (C) 2013 Andrew Trevorrow and Tomas Rokicki.
-
- This program is free software; you can redistribute it and/or
- modify it under the terms of the GNU General Public License
- as published by the Free Software Foundation; either version 2
- of the License, or (at your option) any later version.
-
- This program is distributed in the hope that it will be useful,
- but WITHOUT ANY WARRANTY; without even the implied warranty of
- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- GNU General Public License for more details.
-
- You should have received a copy of the GNU General Public License
- along with this program; if not, write to the Free Software
- Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
-
- Web site:  http://sourceforge.net/projects/golly
- Authors:   rokicki@gmail.com  andrew@trevorrow.com
-
- / ***/
+// This file is part of Golly.
+// See docs/License.html for the copyright notice.
 
 #include "bigint.h"
 #include "lifealgo.h"
 #include "writepattern.h"   // for MC_format, XRLE_format
 
 #include "select.h"         // for Selection
-#include "view.h"           // for OutsideLimits, etc
+#include "view.h"           // for OutsideLimits, nopattupdate, etc
 #include "utils.h"          // for Warning, Fatal, etc
 #include "algos.h"          // for algo_type
 #include "layer.h"          // for currlayer, numclones, MarkLayerDirty, etc
@@ -339,6 +318,9 @@ ChangeNode::~ChangeNode()
 
 void ChangeNode::ChangeCells(bool undo)
 {
+    // avoid possible pattern update during a setcell call (can happen if cellcount is large)
+    nopattupdate = true;
+
     // change state of cell(s) stored in cellinfo array
     if (undo) {
         // we must undo the cell changes in reverse order in case
@@ -356,6 +338,8 @@ void ChangeNode::ChangeCells(bool undo)
         }
     }
     if (cellcount > 0) currlayer->algo->endofpattern();
+
+    nopattupdate = false;
 }
 
 // -----------------------------------------------------------------------------
@@ -903,7 +887,7 @@ void UndoRedo::AddGenChange()
     prevexpo = currlayer->startexpo;
     prevfile.clear();
 
-    // play safe and pretend RememberGenStart was called
+    // pretend RememberGenStart was called
     startcount = 1;
 
     // avoid RememberGenFinish returning early if inscript is true
@@ -918,6 +902,9 @@ void UndoRedo::AddGenChange()
 
 void UndoRedo::SyncUndoHistory()
 {
+    // reset startcount for the next RememberGenStart call
+    startcount = 0;
+
     // synchronize undo history due to a ResetPattern call;
     // wind back the undo list to just past the genchange node that
     // matches the current layer's starting gen count
@@ -950,6 +937,7 @@ void UndoRedo::SyncUndoHistory()
             return;
         }
     }
+    
     // should never get here
     Warning("Bug detected in SyncUndoHistory!");
 }
