@@ -1,26 +1,5 @@
-/*** /
- 
- This file is part of Golly, a Game of Life Simulator.
- Copyright (C) 2013 Andrew Trevorrow and Tomas Rokicki.
- 
- This program is free software; you can redistribute it and/or
- modify it under the terms of the GNU General Public License
- as published by the Free Software Foundation; either version 2
- of the License, or (at your option) any later version.
- 
- This program is distributed in the hope that it will be useful,
- but WITHOUT ANY WARRANTY; without even the implied warranty of
- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- GNU General Public License for more details.
- 
- You should have received a copy of the GNU General Public License
- along with this program; if not, write to the Free Software
- Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
- 
- Web site:  http://sourceforge.net/projects/golly
- Authors:   rokicki@gmail.com  andrew@trevorrow.com
- 
- / ***/
+// This file is part of Golly.
+// See docs/License.html for the copyright notice.
 
 #include "prefs.h"      // for allowundo, SavePrefs
 #include "algos.h"      // for algo_type, NumAlgos, etc
@@ -265,6 +244,8 @@ static void CreateRuleLinks(std::string& htmldata, const std::string& dir,
                          baseURL:[NSURL fileURLWithPath:[[NSBundle mainBundle] bundlePath]]];
                                  // the above base URL is needed for links like <img src='foo.png'> to work
     } else {
+        // replace any spaces with underscores
+        algoname = [algoname stringByReplacingOccurrencesOfString:@" " withString:@"_"];
         NSBundle *bundle=[NSBundle mainBundle];
         NSString *filePath = [bundle pathForResource:algoname
                                               ofType:@"html"
@@ -349,7 +330,7 @@ static bool keepalgoindex = false;
 
 // -----------------------------------------------------------------------------
 
-static int globalButton;
+static NSInteger globalButton;
 
 - (void)doDelayedAction
 {
@@ -359,7 +340,7 @@ static int globalButton;
         curroffset[algoindex] = htmlView.scrollView.contentOffset;
         
         // save chosen algo for later use
-        algoindex = globalButton;
+        algoindex = (algo_type)globalButton;
         
         // display the chosen algo name
         [algoButton setTitle:[NSString stringWithCString:GetAlgoName(algoindex) encoding:NSUTF8StringEncoding]
