@@ -1,34 +1,14 @@
-/*** /
-
- This file is part of Golly, a Game of Life Simulator.
- Copyright (C) 2013 Andrew Trevorrow and Tomas Rokicki.
-
- This program is free software; you can redistribute it and/or
- modify it under the terms of the GNU General Public License
- as published by the Free Software Foundation; either version 2
- of the License, or (at your option) any later version.
-
- This program is distributed in the hope that it will be useful,
- but WITHOUT ANY WARRANTY; without even the implied warranty of
- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- GNU General Public License for more details.
-
- You should have received a copy of the GNU General Public License
- along with this program; if not, write to the Free Software
- Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
-
- Web site:  http://sourceforge.net/projects/golly
- Authors:   rokicki@gmail.com  andrew@trevorrow.com
-
- / ***/
+// This file is part of Golly.
+// See docs/License.html for the copyright notice.
 
 // gollybase
 #include "bigint.h"
 #include "lifealgo.h"
 #include "qlifealgo.h"
 #include "hlifealgo.h"
-#include "jvnalgo.h"
 #include "generationsalgo.h"
+#include "ltlalgo.h"
+#include "jvnalgo.h"
 #include "ruleloaderalgo.h"
 
 // gui-common
@@ -1427,6 +1407,7 @@ void UpdateMenuItems(const char* id)
         jsTickMenuItem("algo2", currlayer->algtype == 2);
         jsTickMenuItem("algo3", currlayer->algtype == 3);
         jsTickMenuItem("algo4", currlayer->algtype == 4);
+        jsTickMenuItem("algo5", currlayer->algtype == 5);
     
     } else if (menu == "View_menu") {
         if (SelectionExists()) {
@@ -1519,6 +1500,7 @@ void DoMenuItem(const char* id)
     if (item == "algo2") SetAlgo(2); else
     if (item == "algo3") SetAlgo(3); else
     if (item == "algo4") SetAlgo(4); else
+    if (item == "algo5") SetAlgo(5); else
     if (item == "control_setrule") SetRule(); else
     
     // items in View menu:
@@ -1973,7 +1955,7 @@ static void DoFrame()
 
 int EMSCRIPTEN_KEEPALIVE main()
 {
-    SetMessage("This is Golly 0.9 for the web.  Copyright 2016 The Golly Gang.");
+    SetMessage("This is Golly 3.2 for the web.  Copyright 2005-2018 The Golly Gang.");
     InitPaths();                // init tempdir, prefsfile, etc
     MAX_MAG = 5;                // maximum cell size = 32x32
     maxhashmem = 300;           // enough for caterpillar
