@@ -1,26 +1,5 @@
-/*** /
- 
- This file is part of Golly, a Game of Life Simulator.
- Copyright (C) 2013 Andrew Trevorrow and Tomas Rokicki.
- 
- This program is free software; you can redistribute it and/or
- modify it under the terms of the GNU General Public License
- as published by the Free Software Foundation; either version 2
- of the License, or (at your option) any later version.
- 
- This program is distributed in the hope that it will be useful,
- but WITHOUT ANY WARRANTY; without even the implied warranty of
- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- GNU General Public License for more details.
- 
- You should have received a copy of the GNU General Public License
- along with this program; if not, write to the Free Software
- Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
- 
- Web site:  http://sourceforge.net/projects/golly
- Authors:   rokicki@gmail.com  andrew@trevorrow.com
- 
- / ***/
+// This file is part of Golly.
+// See docs/License.html for the copyright notice.
 
 #include "wx/wxprec.h"      // for compilers that support precompilation
 #ifndef WX_PRECOMP
@@ -109,7 +88,7 @@ bool MainFrame::SaveStartingPattern()
     }
 
     currlayer->currfile = currlayer->tempstart;     // ResetPattern will load tempstart
-    
+
     // save starting pattern in tempstart file
     if ( currlayer->algo->hyperCapable() ) {
         // much faster to save pattern in a macrocell file
@@ -248,8 +227,7 @@ void MainFrame::ResetPattern(bool resetundo)
     
     if (allowundo && !currlayer->stayclean) {
         if (inscript) {
-            // script called reset() so remember gen change
-            // (RememberGenStart was called above)
+            // script called reset() so remember gen change (RememberGenStart was called above)
             currlayer->undoredo->RememberGenFinish();
         } else if (resetundo) {
             // wind back the undo history to the starting pattern
@@ -569,7 +547,7 @@ void MainFrame::DisplayPattern()
 bool MainFrame::StepPattern()
 {
     lifealgo* curralgo = currlayer->algo;
-    if (curralgo->gridwd > 0 || curralgo->gridht > 0) {
+    if (curralgo->unbounded && (curralgo->gridwd > 0 || curralgo->gridht > 0)) {
         // bounded grid, so temporarily set the increment to 1 so we can call
         // CreateBorderCells() and DeleteBorderCells() around each step()
         int savebase = currlayer->currbase;
@@ -744,7 +722,9 @@ void MainFrame::DoPendingAction(bool restart)
         mouseevent.SetEventObject(viewptr);
         viewptr->GetEventHandler()->ProcessEvent(mouseevent);
         while (viewptr->drawingcells) {
+            insideYield = true;
             wxGetApp().Yield(true);
+            insideYield = false;
             wxMilliSleep(5);             // don't hog CPU
         }
         
@@ -783,7 +763,7 @@ void MainFrame::DisplayTimingInfo()
 
 void MainFrame::StartGenerating()
 {
-    if (viewptr->drawingcells || viewptr->waitingforclick) {
+    if (insideYield || viewptr->drawingcells || viewptr->waitingforclick) {
         return;
     }
     
@@ -874,12 +854,14 @@ void MainFrame::StopGenerating()
 
 // -----------------------------------------------------------------------------
 
+// this flag is used to avoid re-entrancy in OnGenTimer (note that on Windows
+// the timer can fire while a wxMessageBox dialog is open)
+static bool in_timer = false;
+
 void MainFrame::OnGenTimer(wxTimerEvent& WXUNUSED(event))
 {
-    // called when gentimer fires a timer event
-    
-    // play safe and avoid re-entrancy if timer fires while insideYield
-    if (insideYield) return;
+    if (in_timer) return;
+    in_timer = true;
     
     if (!StepPattern()) {
         if (generating) {
@@ -889,6 +871,7 @@ void MainFrame::OnGenTimer(wxTimerEvent& WXUNUSED(event))
             // StopGenerating() was called while insideYield
             FinishUp();
         }
+        in_timer = false;
         return;
     }
     
@@ -905,6 +888,7 @@ void MainFrame::OnGenTimer(wxTimerEvent& WXUNUSED(event))
             wxString msg;
             msg.Printf(_("No more frames can be recorded (maximum = %d)."), MAX_FRAME_COUNT);
             Warning(msg);
+            in_timer = false;
             return;
         }
     } else if (currlayer->hyperspeed && currlayer->algo->hyperCapable()) {
@@ -919,6 +903,8 @@ void MainFrame::OnGenTimer(wxTimerEvent& WXUNUSED(event))
         // StopGenerating() was called while insideYield
         FinishUp();
     }
+    
+    in_timer = false;
 }
 
 // -----------------------------------------------------------------------------
@@ -967,6 +953,12 @@ void MainFrame::NextGeneration(bool useinc)
         return;
     }
     
+    if (insideYield) {
+        // avoid calling step() recursively
+        inNextGen = false;
+        return;
+    }
+    
     if (viewptr->drawingcells || viewptr->waitingforclick) {
         Beep();
         inNextGen = false;
@@ -1033,7 +1025,7 @@ void MainFrame::NextGeneration(bool useinc)
         viewptr->CheckCursor(infront);
     }
     
-    bool boundedgrid = (curralgo->gridwd > 0 || curralgo->gridht > 0);
+    bool boundedgrid = curralgo->unbounded && (curralgo->gridwd > 0 || curralgo->gridht > 0);
     
     if (useinc) {
         // step by current increment
@@ -1135,6 +1127,15 @@ void MainFrame::ToggleHashInfo()
 
 // -----------------------------------------------------------------------------
 
+void MainFrame::ToggleShowPopulation()
+{
+    showpopulation = !showpopulation;
+    
+    if (generating && showstatus) statusptr->Refresh(false);
+}
+
+// -----------------------------------------------------------------------------
+
 void MainFrame::ClearOutsideGrid()
 {
     // check current pattern and clear any live cells outside bounded grid
@@ -1143,84 +1144,102 @@ void MainFrame::ClearOutsideGrid()
     
     // might also need to truncate selection
     currlayer->currsel.CheckGridEdges();
-    
-    if (currlayer->algo->isEmpty()) return;
-    
-    // check if current pattern is too big to use nextcell/setcell
-    bigint top, left, bottom, right;
-    currlayer->algo->findedges(&top, &left, &bottom, &right);
-    if ( viewptr->OutsideLimits(top, left, bottom, right) ) {
-        statusptr->ErrorMessage(_("Pattern too big to check (outside +/- 10^9 boundary)."));
-        return;
-    }
-    
-    int itop = top.toint();
-    int ileft = left.toint();
-    int ibottom = bottom.toint();
-    int iright = right.toint();
-    
-    // no need to do anything if pattern is entirely within grid
-    int gtop = currlayer->algo->gridtop.toint();
-    int gleft = currlayer->algo->gridleft.toint();
-    int gbottom = currlayer->algo->gridbottom.toint();
-    int gright = currlayer->algo->gridright.toint();
-    if (currlayer->algo->gridwd == 0) {
-        // grid has infinite width
-        gleft = INT_MIN;
-        gright = INT_MAX;
-    }
-    if (currlayer->algo->gridht == 0) {
-        // grid has infinite height
-        gtop = INT_MIN;
-        gbottom = INT_MAX;
-    }
-    if (itop >= gtop && ileft >= gleft && ibottom <= gbottom && iright <= gright) {
-        return;
-    }
-    
-    int ht = ibottom - itop + 1;
-    int cx, cy;
-    
-    // for showing accurate progress we need to add pattern height to pop count
-    // in case this is a huge pattern with many blank rows
-    double maxcount = currlayer->algo->getPopulation().todouble() + ht;
-    double accumcount = 0;
-    int currcount = 0;
-    bool abort = false;
-    int v = 0;
-    BeginProgress(_("Checking cells outside grid"));
-    
-    lifealgo* curralgo = currlayer->algo;
-    for ( cy=itop; cy<=ibottom; cy++ ) {
-        currcount++;
-        for ( cx=ileft; cx<=iright; cx++ ) {
-            int skip = curralgo->nextcell(cx, cy, v);
-            if (skip >= 0) {
-                // found next live cell in this row
-                cx += skip;
-                if (cx < gleft || cx > gright || cy < gtop || cy > gbottom) {
-                    // clear cell outside grid
-                    if (savechanges) currlayer->undoredo->SaveCellChange(cx, cy, v, 0);
-                    curralgo->setcell(cx, cy, 0);
-                    patternchanged = true;
+
+    if (!currlayer->algo->unbounded) {
+        if (currlayer->algo->clipped_cells.size() > 0) {
+            // cells outside the grid were clipped
+            if (savechanges) {
+                for (size_t i = 0; i < currlayer->algo->clipped_cells.size(); i += 3) {
+                    int x = currlayer->algo->clipped_cells[i];
+                    int y = currlayer->algo->clipped_cells[i+1];
+                    int s = currlayer->algo->clipped_cells[i+2];
+                    currlayer->undoredo->SaveCellChange(x, y, s, 0);
                 }
-                currcount++;
-            } else {
-                cx = iright;  // done this row
-            }
-            if (currcount > 1024) {
-                accumcount += currcount;
-                currcount = 0;
-                abort = AbortProgress(accumcount / maxcount, wxEmptyString);
-                if (abort) break;
             }
+            currlayer->algo->clipped_cells.clear();
+            patternchanged = true;
         }
-        if (abort) break;
+
+    } else {
+        // algo uses an unbounded grid
+        if (currlayer->algo->isEmpty()) return;
+        
+        // check if current pattern is too big to use nextcell/setcell
+        bigint top, left, bottom, right;
+        currlayer->algo->findedges(&top, &left, &bottom, &right);
+        if ( viewptr->OutsideLimits(top, left, bottom, right) ) {
+            statusptr->ErrorMessage(_("Pattern too big to check (outside +/- 10^9 boundary)."));
+            return;
+        }
+        
+        int itop = top.toint();
+        int ileft = left.toint();
+        int ibottom = bottom.toint();
+        int iright = right.toint();
+        
+        // no need to do anything if pattern is entirely within grid
+        int gtop = currlayer->algo->gridtop.toint();
+        int gleft = currlayer->algo->gridleft.toint();
+        int gbottom = currlayer->algo->gridbottom.toint();
+        int gright = currlayer->algo->gridright.toint();
+        if (currlayer->algo->gridwd == 0) {
+            // grid has infinite width
+            gleft = INT_MIN;
+            gright = INT_MAX;
+        }
+        if (currlayer->algo->gridht == 0) {
+            // grid has infinite height
+            gtop = INT_MIN;
+            gbottom = INT_MAX;
+        }
+        if (itop >= gtop && ileft >= gleft && ibottom <= gbottom && iright <= gright) {
+            return;
+        }
+        
+        int ht = ibottom - itop + 1;
+        int cx, cy;
+        
+        // for showing accurate progress we need to add pattern height to pop count
+        // in case this is a huge pattern with many blank rows
+        double maxcount = currlayer->algo->getPopulation().todouble() + ht;
+        double accumcount = 0;
+        int currcount = 0;
+        bool abort = false;
+        int v = 0;
+        BeginProgress(_("Checking cells outside grid"));
+        
+        lifealgo* curralgo = currlayer->algo;
+        for ( cy=itop; cy<=ibottom; cy++ ) {
+            currcount++;
+            for ( cx=ileft; cx<=iright; cx++ ) {
+                int skip = curralgo->nextcell(cx, cy, v);
+                if (skip >= 0) {
+                    // found next live cell in this row
+                    cx += skip;
+                    if (cx < gleft || cx > gright || cy < gtop || cy > gbottom) {
+                        // clear cell outside grid
+                        if (savechanges) currlayer->undoredo->SaveCellChange(cx, cy, v, 0);
+                        curralgo->setcell(cx, cy, 0);
+                        patternchanged = true;
+                    }
+                    currcount++;
+                } else {
+                    cx = iright;  // done this row
+                }
+                if (currcount > 1024) {
+                    accumcount += currcount;
+                    currcount = 0;
+                    abort = AbortProgress(accumcount / maxcount, wxEmptyString);
+                    if (abort) break;
+                }
+            }
+            if (abort) break;
+        }
+        
+        curralgo->endofpattern();
+        EndProgress();
     }
     
-    curralgo->endofpattern();
-    EndProgress();
-    
     if (patternchanged) {
         statusptr->ErrorMessage(_("Pattern was truncated (live cells were outside grid)."));
     }
