@@ -1,26 +1,5 @@
-/*** /
- 
- This file is part of Golly, a Game of Life Simulator.
- Copyright (C) 2013 Andrew Trevorrow and Tomas Rokicki.
- 
- This program is free software; you can redistribute it and/or
- modify it under the terms of the GNU General Public License
- as published by the Free Software Foundation; either version 2
- of the License, or (at your option) any later version.
- 
- This program is distributed in the hope that it will be useful,
- but WITHOUT ANY WARRANTY; without even the implied warranty of
- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- GNU General Public License for more details.
- 
- You should have received a copy of the GNU General Public License
- along with this program; if not, write to the Free Software
- Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
- 
- Web site:  http://sourceforge.net/projects/golly
- Authors:   rokicki@gmail.com  andrew@trevorrow.com
- 
- / ***/
+// This file is part of Golly.
+// See docs/License.html for the copyright notice.
 
 #include "wx/wxprec.h"      // for compilers that support precompilation
 #ifndef WX_PRECOMP
@@ -40,7 +19,6 @@
 #include "wxutils.h"       // for Fatal
 #include "wxprefs.h"       // for showedit, showallstates, etc
 #include "wxstatus.h"      // for statusptr->...
-#include "wxscript.h"      // for inscript
 #include "wxview.h"        // for viewptr->...
 #include "wxrender.h"      // for DrawOneIcon
 #include "wxlayer.h"       // for currlayer, LayerBarHeight, SetLayerColors
@@ -468,7 +446,7 @@ void EditBar::DrawEditBar(wxDC& dc, int wd, int ht)
     wxBrush brush(wxColor(202,202,202));
     FillRect(dc, r, brush);
 #endif
-    
+
 #ifdef __WXMSW__
     // use theme background color on Windows
     wxBrush brush(GetBackgroundColour());
@@ -590,8 +568,6 @@ void EditBar::OnMouseDown(wxMouseEvent& event)
     mainptr->showbanner = false;
     statusptr->ClearMessage();
     
-    if (inscript) return;    // let script control drawing state
-    
     int x = event.GetX();
     int y = event.GetY();
     
@@ -917,6 +893,11 @@ void UpdateEditBar()
         
         // drawing state might have changed
         editbarptr->UpdateScrollBar();
+        
+        #ifdef __WXGTK__
+            // avoid bug that can cause buttons to lose their bitmaps
+            editbarptr->Update();
+        #endif
     }
 }
 
@@ -983,5 +964,9 @@ void CycleDrawingState(bool higher)
     if (showedit) {
         editbarptr->Refresh(false);
         editbarptr->UpdateScrollBar();
+        #ifdef __WXGTK__
+            // avoid bug that can cause buttons to lose their bitmaps
+            editbarptr->Update();
+        #endif
     }
 }
