@@ -1,26 +1,5 @@
-/*** /
- 
- This file is part of Golly, a Game of Life Simulator.
- Copyright (C) 2013 Andrew Trevorrow and Tomas Rokicki.
- 
- This program is free software; you can redistribute it and/or
- modify it under the terms of the GNU General Public License
- as published by the Free Software Foundation; either version 2
- of the License, or (at your option) any later version.
- 
- This program is distributed in the hope that it will be useful,
- but WITHOUT ANY WARRANTY; without even the implied warranty of
- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- GNU General Public License for more details.
- 
- You should have received a copy of the GNU General Public License
- along with this program; if not, write to the Free Software
- Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
- 
- Web site:  http://sourceforge.net/projects/golly
- Authors:   rokicki@gmail.com  andrew@trevorrow.com
- 
- / ***/
+// This file is part of Golly.
+// See docs/License.html for the copyright notice.
 
 #include "wx/wxprec.h"     // for compilers that support precompilation
 #ifndef WX_PRECOMP
@@ -49,11 +28,12 @@
 #include "wxinfo.h"        // for GetInfoFrame
 #include "wxstatus.h"      // for statusptr->...
 #include "wxview.h"        // for viewptr->...
-#include "wxscript.h"      // for RunScript, inscript
+#include "wxscript.h"      // for RunScript, inscript, scripttitle
 #include "wxmain.h"        // for MainFrame, etc
 #include "wxundo.h"        // for currlayer->undoredo->...
 #include "wxalgos.h"       // for CreateNewUniverse, algo_type, algoinfo, etc
 #include "wxlayer.h"       // for currlayer, etc
+#include "wxoverlay.h"     // for curroverlay
 #include "wxhelp.h"        // for ShowHelp, LoadRule
 #include "wxtimeline.h"    // for InitTimelineFrame, ToggleTimelineBar, etc
 
@@ -84,6 +64,11 @@ wxString MainFrame::GetBaseName(const wxString& path)
 
 void MainFrame::SetWindowTitle(const wxString& filename)
 {
+    if ( !scripttitle.IsEmpty() ) {
+        // script has called settitle command
+        return;
+    }
+
     if ( !filename.IsEmpty() ) {
         // remember current file name
         currlayer->currname = filename;
@@ -311,7 +296,7 @@ void MainFrame::LoadPattern(const wxString& path, const wxString& newtitle,
     delete currlayer->algo;
     currlayer->algo = CreateNewUniverse(currlayer->algtype);
     
-    if (!newtitle.IsEmpty()) {
+    if (!newtitle.IsEmpty() && !inscript) {
         // show new file name in window title but no rule (which readpattern can change);
         // nicer if user can see file name while loading a very large pattern
         SetTitle(_("Loading ") + newtitle);
@@ -554,7 +539,10 @@ void MainFrame::OpenZipFile(const wxString& zippath)
     wxSortedArrayString deplist;            // list of installed deprecated files
     wxSortedArrayString rulelist;           // list of installed .rule files
     
-    wxString contents = wxT("<html><title>") + GetBaseName(zippath);
+    // note that we need to create the HTML file with UTF-8 encoding
+    // in case zippath contains non-ASCII characters
+    wxString contents = wxT("<html><meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">");
+    contents += wxT("<title>") + GetBaseName(zippath);
     contents += wxT("</title>\n");
     contents += wxT("<body bgcolor=\"#FFFFCE\">\n");
     contents += wxT("<p>\n");
@@ -771,13 +759,13 @@ void MainFrame::OpenFile(const wxString& path, bool remember)
         ShowHelp(path);
         return;
     }
-    
+
     if (IsTextFile(path)) {
         // open text file in user's preferred text editor
         EditFile(path);
         return;
     }
-    
+
     if (generating) {
         command_pending = true;
         // assume remember is true (should only be false if called from a script)
@@ -791,22 +779,50 @@ void MainFrame::OpenFile(const wxString& path, bool remember)
         Stop();
         return;
     }
+
+    // note that pass_file_events is false if OpenFile was called from GSF_open
+    if (inscript && pass_file_events) {
+        // ensure path is a full path
+        wxString newpath = path;
+        wxFileName fname(newpath);
+        if (!fname.IsAbsolute()) newpath = gollydir + path;
+        PassFileToScript(newpath);
+        return;
+    }
     
+    if (!inscript && path.EndsWith(wxT(".rle3"))) {
+        if (remember) AddRecentPattern(path);
+        
+        // create a full path in rle3path for GSF_getevent to use
+        rle3path = path;
+        wxFileName fname(rle3path);
+        if (!fname.IsAbsolute()) rle3path = gollydir + path;
+        
+        // start up 3D.lua (it will get a file event containing rle3path)
+        wxString path3D = gollydir + wxT("Scripts");
+        path3D += wxFILE_SEP_PATH;
+        path3D += wxT("Lua");
+        path3D += wxFILE_SEP_PATH;
+        path3D += wxT("3D.lua");
+        RunScript(path3D);
+        return;
+    }
+
     if (IsScriptFile(path)) {
         // execute script
         if (remember) AddRecentScript(path);
         RunScript(path);
-        
+
     } else if (IsZipFile(path)) {
         // process zip file
-        if (remember) AddRecentPattern(path);     // treat it like a pattern
+        if (remember) AddRecentPattern(path);   // treat it like a pattern
         OpenZipFile(path);
-    
+
     } else if (IsRuleFile(path)) {
         // switch to rule, but only if it's in rulesdir or userrules
         if (RuleCanBeFound(path))
             LoadRule(path.AfterLast(wxFILE_SEP_PATH).BeforeLast('.'));
-        
+
     } else {
         // load pattern
         if (remember) AddRecentPattern(path);
@@ -964,6 +980,7 @@ void MainFrame::OpenPattern()
     
     wxString filetypes = _("All files (*)|*");
     filetypes +=         _("|RLE (*.rle)|*.rle");
+    filetypes +=         _("|RLE3 (*.rle3)|*.rle3");
     filetypes +=         _("|Macrocell (*.mc)|*.mc");
     filetypes +=         _("|Gzip (*.gz)|*.gz");
     filetypes +=         _("|Life 1.05/1.06 (*.lif)|*.lif");
@@ -1159,6 +1176,44 @@ bool MainFrame::ClipboardContainsRule()
 
 // -----------------------------------------------------------------------------
 
+bool MainFrame::ClipboardContainsRLE3()
+{
+    wxTextDataObject data;
+    if (!GetTextFromClipboard(&data)) return false;
+
+    wxString cliptext = data.GetText();
+    if (!cliptext.StartsWith(wxT("3D version"))) return false;
+
+    // copy clipboard data to a temporary .rle3 file
+    wxString filepath = tempdir + wxT("clipboard.rle3");
+    wxFile rle3file(filepath, wxFile::write);
+    if (!rle3file.IsOpened()) {
+        Warning(_("Could not open clipboard.rle3 for writing!"));
+        return true;
+    }
+    if (!rle3file.Write(data.GetText())) {
+        Warning(_("Could not write clipboard data to file!"));
+        rle3file.Close();
+        return true;
+    }
+    rle3file.Close();
+
+    // set rle3path for GSF_getevent
+    rle3path = filepath;
+
+    // start up 3D.lua (it will get a file event containing rle3path)
+    wxString path3D = gollydir + wxT("Scripts");
+    path3D += wxFILE_SEP_PATH;
+    path3D += wxT("Lua");
+    path3D += wxFILE_SEP_PATH;
+    path3D += wxT("3D.lua");
+    RunScript(path3D);
+
+    return true;
+}
+
+// -----------------------------------------------------------------------------
+
 void MainFrame::OpenClipboard()
 {
     if (generating) {
@@ -1172,6 +1227,10 @@ void MainFrame::OpenClipboard()
     // and switch to that rule
     if (ClipboardContainsRule()) return;
     
+    // if clipboard text starts with "3D version" then start up 3D.lua
+    // and load the RLE3 pattern
+    if (ClipboardContainsRLE3()) return;
+    
     // load and view pattern data stored in clipboard
     wxTextDataObject data;
     if (GetTextFromClipboard(&data)) {
@@ -1205,7 +1264,7 @@ wxString MainFrame::GetScriptFileName(const wxString& text)
     int linelen = 0;
     
     // need to be careful converting Unicode wxString to char*
-    wxCharBuffer buff = text.mb_str(wxConvLocal);
+    wxCharBuffer buff = text.mb_str(wxConvUTF8);
     const char* p = (const char*) buff;
     while (*p) {
         switch (*p) {
@@ -1705,28 +1764,42 @@ void MainFrame::SaveSucceeded(const wxString& path)
 
 // -----------------------------------------------------------------------------
 
+void MainFrame::SaveOverlay()
+{
+    if (showoverlay && curroverlay->GetOverlayData()) {
+        wxFileDialog savedlg(this, _("Save overlay as PNG file"),
+                             overlaydir, _("overlay.png"), _("PNG (*.png)|*.png"),
+                             wxFD_SAVE | wxFD_OVERWRITE_PROMPT);
+        
+        if (savedlg.ShowModal() == wxID_OK) {
+            wxString pngpath = savedlg.GetPath();
+            wxFileName fullpath(pngpath);
+            overlaydir = fullpath.GetPath();
+            curroverlay->SaveOverlay(pngpath);
+        }
+    }
+}
+
+// -----------------------------------------------------------------------------
+
+static wxString oldfiledir;
+
 void MainFrame::ToggleShowFiles()
 {
     if (splitwin->IsSplit()) dirwinwd = splitwin->GetSashPosition();
-#ifndef __WXMAC__
-    // hide scroll bars
-    bigview->SetScrollbar(wxHORIZONTAL, 0, 0, 0, true);
-    bigview->SetScrollbar(wxVERTICAL, 0, 0, 0, true);
-#endif
-    
     showfiles = !showfiles;
     if (splitwin->IsSplit()) {
         // hide left pane
         splitwin->Unsplit(filectrl);
+        oldfiledir = filedir;
     } else {
         splitwin->SplitVertically(filectrl, RightPane(), dirwinwd);
     }
+    if (showfiles && filedir != oldfiledir) {
+        // show new file directory
+        SimplifyTree(filedir, filectrl->GetTreeCtrl(), filectrl->GetRootId());
+    }
     viewptr->SetFocus();
-
-#ifndef __WXMAC__
-    // restore scroll bars
-    bigview->UpdateScrollBars();
-#endif
 }
 
 // -----------------------------------------------------------------------------
@@ -1734,8 +1807,9 @@ void MainFrame::ToggleShowFiles()
 void MainFrame::ChangeFileDir()
 {
     wxDirDialog dirdlg(this, _("Choose a new file folder"), filedir, wxDD_NEW_DIR_BUTTON);
-    if (dirdlg.ShowModal() == wxID_OK)
+    if (dirdlg.ShowModal() == wxID_OK) {
         SetFileDir(dirdlg.GetPath());
+    }
 }
 
 // -----------------------------------------------------------------------------
