@@ -1,26 +1,5 @@
-/*** /
- 
- This file is part of Golly, a Game of Life Simulator.
- Copyright (C) 2013 Andrew Trevorrow and Tomas Rokicki.
- 
- This program is free software; you can redistribute it and/or
- modify it under the terms of the GNU General Public License
- as published by the Free Software Foundation; either version 2
- of the License, or (at your option) any later version.
- 
- This program is distributed in the hope that it will be useful,
- but WITHOUT ANY WARRANTY; without even the implied warranty of
- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- GNU General Public License for more details.
- 
- You should have received a copy of the GNU General Public License
- along with this program; if not, write to the Free Software
- Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
- 
- Web site:  http://sourceforge.net/projects/golly
- Authors:   rokicki@gmail.com  andrew@trevorrow.com
- 
- / ***/
+// This file is part of Golly.
+// See docs/License.html for the copyright notice.
 
 // A GUI for Golly, implemented in wxWidgets (www.wxwidgets.org).
 // Unfinished code is flagged by "!!!".
@@ -334,12 +313,16 @@ bool GollyApp::OnInit()
     
     wxString banner = _("This is Golly version ");
     banner +=         _(STRINGIFY(VERSION)); 
+    banner +=         _(" (");
 #ifdef GOLLY64BIT
-    banner +=         _(" (64-bit)");
+    banner +=         _("64-bit");
 #else
-    banner +=         _(" (32-bit)");
+    banner +=         _("32-bit");
 #endif
-    banner +=         _(".  Copyright 2016 The Golly Gang.");
+#ifdef ENABLE_SOUND
+    banner +=         _(", Sound");
+#endif
+    banner +=         _(").  Copyright 2005-2018 The Golly Gang.");
     if (debuglevel > 0) {
         banner += wxString::Format(_("  *** debuglevel = %d ***"), debuglevel);
     }
