@@ -1,26 +1,5 @@
-/*** /
- 
- This file is part of Golly, a Game of Life Simulator.
- Copyright (C) 2013 Andrew Trevorrow and Tomas Rokicki.
- 
- This program is free software; you can redistribute it and/or
- modify it under the terms of the GNU General Public License
- as published by the Free Software Foundation; either version 2
- of the License, or (at your option) any later version.
- 
- This program is distributed in the hope that it will be useful,
- but WITHOUT ANY WARRANTY; without even the implied warranty of
- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- GNU General Public License for more details.
- 
- You should have received a copy of the GNU General Public License
- along with this program; if not, write to the Free Software
- Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
- 
- Web site:  http://sourceforge.net/projects/golly
- Authors:   rokicki@gmail.com  andrew@trevorrow.com
- 
- / ***/
+// This file is part of Golly.
+// See docs/License.html for the copyright notice.
 
 #include "wx/wxprec.h"     // for compilers that support precompilation
 #ifndef WX_PRECOMP
@@ -41,7 +20,7 @@
 #include "wxview.h"        // for viewptr->...
 #include "wxutils.h"       // for Warning, BeginProgress, etc
 #include "wxprefs.h"       // for GetShortcutTable, helpfontsize, gollydir, etc
-#include "wxscript.h"      // for inscript
+#include "wxscript.h"      // for inscript, pass_file_events, etc
 #include "wxlayer.h"       // for numlayers, GetLayer, etc
 #include "wxalgos.h"       // for QLIFE_ALGO
 #include "wxhelp.h"
@@ -237,7 +216,7 @@ HelpFrame::HelpFrame()
     
     wxButton* closebutt = new wxButton(this, wxID_CLOSE, _("Close"));
     closebutt->SetDefault();
-    hbox->Add(closebutt, 0, wxALL | wxALIGN_RIGHT, 10);
+    hbox->Add(closebutt, 0, wxALL, 10);
     
     vbox->Add(hbox, 0, wxALL | wxEXPAND | wxALIGN_TOP, 0);
     
@@ -271,6 +250,20 @@ void UpdateHelpButtons()
     
     wxString location = htmlwin->GetOpenedPage();
     if ( !location.IsEmpty() ) {
+
+        if (location.StartsWith(wxT("file:"))) {
+            // this happens in wx 3.1.0, so convert location from URL to file path
+            wxFileName fname = wxFileSystem::URLToFileName(location);
+            location = fname.GetFullPath();
+            #ifdef __WXMSW__
+                location.Replace(wxT("\\"), wxT("/"));
+            #endif
+        }
+        
+        // avoid bug in wx 3.1.0
+        location.Replace(wxT("%20"), wxT(" "));
+        location.Replace(wxT("%23"), wxT("#"));
+        
         // set currhelp so user can close help window and then open same page
         currhelp = location;
         
@@ -530,23 +523,23 @@ void LoadRule(const wxString& rulestring, bool fromfile)
         // show new rule in main window's title but don't change name
         mainptr->SetWindowTitle(wxEmptyString);
         
-		// if pattern exists and is at starting gen then ensure savestart is true
-		// so that SaveStartingPattern will save pattern to suitable file
-		// (and thus undo/reset will work correctly)
-		if (currlayer->algo->getGeneration() == currlayer->startgen && !currlayer->algo->isEmpty()) {
-			currlayer->savestart = true;
-		}
+        // if pattern exists and is at starting gen then ensure savestart is true
+        // so that SaveStartingPattern will save pattern to suitable file
+        // (and thus undo/reset will work correctly)
+        if (currlayer->algo->getGeneration() == currlayer->startgen && !currlayer->algo->isEmpty()) {
+            currlayer->savestart = true;
+        }
         
         // if grid is bounded then remove any live cells outside grid edges
         if (currlayer->algo->gridwd > 0 || currlayer->algo->gridht > 0) {
             mainptr->ClearOutsideGrid();
         }
-	
-		// new rule might have changed the number of cell states;
-		// if there are fewer states then pattern might change
-		if (newmaxstate < oldmaxstate && !currlayer->algo->isEmpty()) {
-			mainptr->ReduceCellStates(newmaxstate);
-		}
+    
+        // new rule might have changed the number of cell states;
+        // if there are fewer states then pattern might change
+        if (newmaxstate < oldmaxstate && !currlayer->algo->isEmpty()) {
+            mainptr->ReduceCellStates(newmaxstate);
+        }
 
         if (allowundo && !currlayer->stayclean) {
             currlayer->undoredo->RememberRuleChange(oldrule);
@@ -659,6 +652,7 @@ void GetURL(const wxString& url)
     }
     
     wxString filename = fullurl.AfterLast('/');
+    
     // remove ugly stuff at start of file names downloaded from ConwayLife.com
     if (filename.StartsWith(wxT("download.php?f=")) ||
         filename.StartsWith(wxT("pattern.asp?p=")) ||
@@ -711,9 +705,7 @@ void GetURL(const wxString& url)
     }
     
     if (IsHTMLFile(filename)) {
-        // display html file in help window;
-        // search file for any simple img links and also download those files???
-        // maybe in version 3!
+        // display html file in help window
         htmlwin->LoadPage(filepath);
         
     } else if (IsRuleFile(filename)) {
@@ -963,24 +955,22 @@ void HtmlView::OnLinkClicked(const wxHtmlLinkInfo& link)
         }
         
     } else if ( url.StartsWith(wxT("prefs:")) ) {
-        if (inscript) {
-            Warning(_("Cannot change preferences while a script is running."));
-        } else {
-            // user clicked on link to Preferences dialog
-            mainptr->ShowPrefsDialog( url.AfterFirst(':') );
-        }
+        // user clicked on link to Preferences dialog
+        mainptr->ShowPrefsDialog( url.AfterFirst(':') );
         
     } else if ( url.StartsWith(wxT("open:")) ) {
+        wxString path = url.AfterFirst(':');
+#ifdef __WXMSW__
+        path.Replace(wxT("/"), wxT("\\"));
+#endif
+        wxFileName fname(path);
+        if (!fname.IsAbsolute()) path = gollydir + path;
         if (inscript) {
-            Warning(_("Cannot open file while a script is running."));
+            if (pass_file_events) {
+                PassFileToScript(path);
+            }
         } else {
             // open clicked file
-            wxString path = url.AfterFirst(':');
-#ifdef __WXMSW__
-            path.Replace(wxT("/"), wxT("\\"));
-#endif
-            wxFileName fname(path);
-            if (!fname.IsAbsolute()) path = gollydir + path;
             if (editlink) {
                 mainptr->EditFile(path);
             } else {
@@ -1095,7 +1085,17 @@ void HtmlView::CheckAndLoad(const wxString& filepath)
         
     } else {
         // assume full path or local link
-        LoadPage(filepath);
+        #if defined(__WXMSW__) && wxCHECK_VERSION(3,1,0)
+            wxFileName fname(filepath);
+            if (fname.IsAbsolute()) {
+                // call LoadFile rather than LoadPage to avoid bug in wx 3.1.0 on Windows 10
+                LoadFile(fname);
+            } else {
+                LoadPage(filepath);
+            }
+        #else
+            LoadPage(filepath);
+        #endif
     }
 }
 
@@ -1221,9 +1221,23 @@ void HtmlView::OnSize(wxSizeEvent& event)
     
     wxHtmlWindow::OnSize(event);
     
-    wxString currpage = GetOpenedPage();
-    if ( !currpage.IsEmpty() && canreload ) {
-        CheckAndLoad(currpage);    // reload page
+    wxString location = GetOpenedPage();
+    if ( !location.IsEmpty() && canreload ) {
+
+        if (location.StartsWith(wxT("file:"))) {
+            // this happens in wx 3.1.0, so convert location from URL to file path
+            wxFileName fname = wxFileSystem::URLToFileName(location);
+            location = fname.GetFullPath();
+            #ifdef __WXMSW__
+                location.Replace(wxT("\\"), wxT("/"));
+            #endif
+        }
+        
+        // avoid bug in wx 3.1.0
+        location.Replace(wxT("%20"), wxT(" "));
+        location.Replace(wxT("%23"), wxT("#"));
+    
+        CheckAndLoad(location);    // reload page
         Scroll(x, y);              // scroll to old position
     }
     
