@@ -1,26 +1,5 @@
-/*** /
- 
- This file is part of Golly, a Game of Life Simulator.
- Copyright (C) 2013 Andrew Trevorrow and Tomas Rokicki.
- 
- This program is free software; you can redistribute it and/or
- modify it under the terms of the GNU General Public License
- as published by the Free Software Foundation; either version 2
- of the License, or (at your option) any later version.
- 
- This program is distributed in the hope that it will be useful,
- but WITHOUT ANY WARRANTY; without even the implied warranty of
- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- GNU General Public License for more details.
- 
- You should have received a copy of the GNU General Public License
- along with this program; if not, write to the Free Software
- Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
- 
- Web site:  http://sourceforge.net/projects/golly
- Authors:   rokicki@gmail.com  andrew@trevorrow.com
- 
- / ***/
+// This file is part of Golly.
+// See docs/License.html for the copyright notice.
 
 /*
     Golly uses a statically embedded Lua interpreter to execute .lua scripts.
@@ -62,18 +41,19 @@
 #include "readpattern.h"
 #include "writepattern.h"
 
-#include "wxgolly.h"        // for wxGetApp, mainptr, viewptr, statusptr
+#include "wxgolly.h"        // for wxGetApp, mainptr, viewptr, statusptr, stopwatch
 #include "wxmain.h"         // for mainptr->...
 #include "wxselect.h"       // for Selection
 #include "wxview.h"         // for viewptr->...
 #include "wxstatus.h"       // for statusptr->...
-#include "wxutils.h"        // for Warning, Note, GetString, etc
+#include "wxutils.h"        // for Warning, Note, GetString, SaveChanges, etc
 #include "wxprefs.h"        // for gollydir, etc
 #include "wxinfo.h"         // for ShowInfo
 #include "wxhelp.h"         // for ShowHelp
 #include "wxundo.h"         // for currlayer->undoredo->...
 #include "wxalgos.h"        // for *_ALGO, CreateNewUniverse, etc
 #include "wxlayer.h"        // for AddLayer, currlayer, currindex, etc
+#include "wxoverlay.h"      // for curroverlay->...
 #include "wxscript.h"       // for inscript, abortmsg, GSF_*, etc
 #include "wxlua.h"
 
@@ -83,8 +63,8 @@
 
 // some useful macros
 
-#define CHECK_RGB(r,g,b,cmd)                                                 \
-    if (r < 0 || r > 255 || g < 0 || g > 255 || g < 0 || g > 255) {         \
+#define CHECK_RGB(r,g,b,cmd)                                                \
+    if (r < 0 || r > 255 || g < 0 || g > 255 || b < 0 || b > 255) {         \
         char msg[128];                                                      \
         sprintf(msg, "%s error: bad rgb value (%d,%d,%d)", cmd, r, g, b);   \
         GollyError(L, msg);                                                 \
@@ -97,12 +77,8 @@
     #define FILENAME filename
 #endif
 
-#ifdef __WXMSW__
-    // encoding conversion from Lua string to wxString is different on Windows
-    #define LUA_ENC wxConvLocal
-#else
-    #define LUA_ENC wxConvUTF8
-#endif
+// use UTF8 for the encoding conversion from Lua string to wxString
+#define LUA_ENC wxConvUTF8
 
 // -----------------------------------------------------------------------------
 
@@ -136,15 +112,23 @@ static void GollyError(lua_State* L, const char* errmsg)
 
 // -----------------------------------------------------------------------------
 
+static bool CheckBoolean(lua_State* L, int arg)
+{
+    luaL_checktype(L, arg, LUA_TBOOLEAN);
+    return lua_toboolean(L, arg) ? true : false;
+}
+
+// -----------------------------------------------------------------------------
+
 static int g_open(lua_State* L)
 {
     CheckEvents(L);
 
     const char* filename = luaL_checkstring(L, 1);
-    int remember = 0;
-    if (lua_gettop(L) > 1) remember = lua_toboolean(L, 2) ? 1 : 0;
+    bool remember = false;
+    if (lua_gettop(L) > 1) remember = CheckBoolean(L, 2);
 
-    const char* err = GSF_open(wxString(filename, LUA_ENC), remember);
+    const char* err = GSF_open(wxString(filename, LUA_ENC), remember ? 1 : 0);
     if (err) GollyError(L, err);
     
     return 0;   // no result
@@ -158,10 +142,10 @@ static int g_save(lua_State* L)
 
     const char* filename = luaL_checkstring(L, 1);
     const char* format = luaL_checkstring(L, 2);
-    int remember = 0;
-    if (lua_gettop(L) > 2) remember = lua_toboolean(L, 3) ? 1 : 0;
+    bool remember = false;
+    if (lua_gettop(L) > 2) remember = CheckBoolean(L, 3);
     
-    const char* err = GSF_save(wxString(filename, LUA_ENC), format, remember);
+    const char* err = GSF_save(wxString(filename, LUA_ENC), format, remember ? 1 : 0);
     if (err) GollyError(L, err);
     
     return 0;   // no result
@@ -177,13 +161,13 @@ static int g_opendialog(lua_State* L)
     const char* filetypes = "All files (*)|*";
     const char* initialdir = "";
     const char* initialfname = "";
-    int mustexist = 1;
+    bool mustexist = true;
     
     if (lua_gettop(L) > 0) title = luaL_checkstring(L, 1);
     if (lua_gettop(L) > 1) filetypes = luaL_checkstring(L, 2);
     if (lua_gettop(L) > 2) initialdir = luaL_checkstring(L, 3);
     if (lua_gettop(L) > 3) initialfname = luaL_checkstring(L, 4);
-    if (lua_gettop(L) > 4) mustexist = lua_toboolean(L, 5) ? 1 : 0;
+    if (lua_gettop(L) > 4) mustexist = CheckBoolean(L, 5);
 
     wxString wxs_title(title, LUA_ENC);
     wxString wxs_filetypes(filetypes, LUA_ENC);
@@ -203,11 +187,11 @@ static int g_opendialog(lua_State* L)
     } else {
         // let user choose a file
         wxFileDialog opendlg(NULL, wxs_title, wxs_initialdir, wxs_initialfname, wxs_filetypes,
-                             wxFD_OPEN | (mustexist == 0 ? 0 : wxFD_FILE_MUST_EXIST) );
+                             wxFD_OPEN | (mustexist ? wxFD_FILE_MUST_EXIST : 0) );
         if (opendlg.ShowModal() == wxID_OK) wxs_result = opendlg.GetPath();
     }
     
-    lua_pushstring(L, (const char*)wxs_result.mb_str(wxConvUTF8));
+    lua_pushstring(L, (const char*)wxs_result.mb_str(LUA_ENC));
     
     return 1;   // result is a string
 }
@@ -222,13 +206,13 @@ static int g_savedialog(lua_State* L)
     const char* filetypes = "All files (*)|*";
     const char* initialdir = "";
     const char* initialfname = "";
-    int suppressprompt = 0;
+    bool suppressprompt = false;
     
     if (lua_gettop(L) > 0) title = luaL_checkstring(L, 1);
     if (lua_gettop(L) > 1) filetypes = luaL_checkstring(L, 2);
     if (lua_gettop(L) > 2) initialdir = luaL_checkstring(L, 3);
     if (lua_gettop(L) > 3) initialfname = luaL_checkstring(L, 4);
-    if (lua_gettop(L) > 4) suppressprompt = lua_toboolean(L, 5) ? 1 : 0;
+    if (lua_gettop(L) > 4) suppressprompt = CheckBoolean(L, 5);
 
     wxString wxs_title(title, LUA_ENC);
     wxString wxs_filetypes(filetypes, LUA_ENC);
@@ -238,11 +222,11 @@ static int g_savedialog(lua_State* L)
     if (wxs_initialdir.IsEmpty()) wxs_initialdir = wxFileName::GetCwd();
     
     wxFileDialog savedlg(NULL, wxs_title, wxs_initialdir, wxs_initialfname, wxs_filetypes,
-                         wxFD_SAVE | (suppressprompt == 0 ? wxFD_OVERWRITE_PROMPT : 0));
+                         wxFD_SAVE | (suppressprompt ? 0 : wxFD_OVERWRITE_PROMPT));
     wxString wxs_savefname = wxEmptyString;
     if (savedlg.ShowModal() == wxID_OK) wxs_savefname = savedlg.GetPath();
     
-    lua_pushstring(L, (const char*)wxs_savefname.mb_str(wxConvUTF8));
+    lua_pushstring(L, (const char*)wxs_savefname.mb_str(LUA_ENC));
     
     return 1;   // result is a string
 }
@@ -452,7 +436,7 @@ static int g_getfiles(lua_State* L)
         // get all files
         bool more = dir.GetFirst(&filename, wxEmptyString, wxDIR_FILES | wxDIR_HIDDEN);
         while (more) {
-            lua_pushstring(L, (const char*)filename.mb_str(wxConvUTF8));
+            lua_pushstring(L, (const char*)filename.mb_str(LUA_ENC));
             lua_rawseti(L, -2, ++arraylen);
             more = dir.GetNext(&filename);
         }
@@ -460,7 +444,7 @@ static int g_getfiles(lua_State* L)
         more = dir.GetFirst(&filename, wxEmptyString, wxDIR_DIRS | wxDIR_HIDDEN);
         while (more) {
             filename += wxFILE_SEP_PATH;
-            lua_pushstring(L, (const char*)filename.mb_str(wxConvUTF8));
+            lua_pushstring(L, (const char*)filename.mb_str(LUA_ENC));
             lua_rawseti(L, -2, ++arraylen);
             more = dir.GetNext(&filename);
         }
@@ -471,6 +455,28 @@ static int g_getfiles(lua_State* L)
 
 // -----------------------------------------------------------------------------
 
+static int g_getpath(lua_State* L)
+{
+    CheckEvents(L);
+
+    lua_pushstring(L, GSF_getpath());
+    
+    return 1;   // result is a string
+}
+
+// -----------------------------------------------------------------------------
+
+static int g_getinfo(lua_State* L)
+{
+    CheckEvents(L);
+
+    lua_pushstring(L, GSF_getinfo());
+    
+    return 1;   // result is a string
+}
+
+// -----------------------------------------------------------------------------
+
 static int g_new(lua_State* L)
 {
     CheckEvents(L);
@@ -557,7 +563,7 @@ static int g_shrink(lua_State* L)
     CheckEvents(L);
     
     bool remove_if_empty = false;
-    if (lua_gettop(L) > 0) remove_if_empty = lua_toboolean(L, 1) ? true : false;
+    if (lua_gettop(L) > 0) remove_if_empty = CheckBoolean(L, 1);
 
     if (viewptr->SelectionExists()) {
         currlayer->currsel.Shrink(false, remove_if_empty);
@@ -840,7 +846,7 @@ static int g_evolve(lua_State* L)
     
     // advance pattern by ngens
     mainptr->generating = true;
-    if (tempalgo->gridwd > 0 || tempalgo->gridht > 0) {
+    if (tempalgo->unbounded && (tempalgo->gridwd > 0 || tempalgo->gridht > 0)) {
         // a bounded grid must use an increment of 1 so we can call
         // CreateBorderCells and DeleteBorderCells around each step()
         tempalgo->setIncrement(1);
@@ -1093,6 +1099,76 @@ static int g_getcells(lua_State* L)
 
 // -----------------------------------------------------------------------------
 
+// maybe only use algo->getcells method if algo is hash-based???!!!
+// (needs more thought and more testing)
+
+#if 0
+
+static int g_getcells2(lua_State* L)
+{
+    CheckEvents(L);
+
+    luaL_checktype(L, 1, LUA_TTABLE);   // rect array with 0 or 4 ints
+
+    lua_newtable(L);
+    int arraylen = 0;
+    
+    int numints = luaL_len(L, 1);
+    if (numints == 0) {
+        // return empty cell array
+    } else if (numints == 4) {
+        lua_rawgeti(L, 1, 1); int ileft = luaL_checkinteger(L,-1); lua_pop(L,1);
+        lua_rawgeti(L, 1, 2); int itop  = luaL_checkinteger(L,-1); lua_pop(L,1);
+        lua_rawgeti(L, 1, 3); int wd    = luaL_checkinteger(L,-1); lua_pop(L,1);
+        lua_rawgeti(L, 1, 4); int ht    = luaL_checkinteger(L,-1); lua_pop(L,1);
+        
+        const char* err = GSF_checkrect(ileft, itop, wd, ht);
+        if (err) GollyError(L, err);
+        
+        int iright = ileft + wd - 1;
+        int ibottom = itop + ht - 1;
+        int cx, cy;
+        lifealgo* curralgo = currlayer->algo;
+        bool multistate = curralgo->NumCellStates() > 2;
+        
+        unsigned char* cellmem = (unsigned char*) malloc(wd * ht);
+        if (cellmem == NULL) {
+            GollyError(L, "getcells error: not enough memory.");
+        }
+        curralgo->getcells(cellmem, ileft, itop, wd, ht);
+        unsigned char* cellptr = cellmem;
+        
+        for ( cy=itop; cy<=ibottom; cy++ ) {
+            for ( cx=ileft; cx<=iright; cx++ ) {
+                unsigned char state = *cellptr++;
+                if (state > 0) {
+                    // found next live cell
+                    lua_pushinteger(L, cx); lua_rawseti(L, -2, ++arraylen);
+                    lua_pushinteger(L, cy); lua_rawseti(L, -2, ++arraylen);
+                    if (multistate) {
+                        lua_pushinteger(L, state); lua_rawseti(L, -2, ++arraylen);
+                    }
+                }
+            }
+        }
+        if (multistate && arraylen > 0 && (arraylen & 1) == 0) {
+            // add padding zero
+            lua_pushinteger(L, 0); lua_rawseti(L, -2, ++arraylen);
+        }
+        
+        free(cellmem);
+        
+    } else {
+        GollyError(L, "getcells error: array must be {} or {x,y,wd,ht}.");
+    }
+    
+    return 1;   // result is a cell array
+}
+
+#endif // #if 0
+
+// -----------------------------------------------------------------------------
+
 static int g_join(lua_State* L)
 {
     CheckEvents(L);
@@ -1850,9 +1926,9 @@ static int g_setview(lua_State* L)
     if (currwd < 0) currwd = 0;
     if (currht < 0) currht = 0;
     
-    int mainwd, mainht;
-    mainptr->GetSize(&mainwd, &mainht);
-    mainptr->SetSize(mainwd + (wd - currwd), mainht + (ht - currht));
+    int mnwd, mnht;
+    mainptr->GetSize(&mnwd, &mnht);
+    mainptr->SetSize(mnwd + (wd - currwd), mnht + (ht - currht));
     
     return 0;   // no result
 }
@@ -1863,15 +1939,35 @@ static int g_getview(lua_State* L)
 {
     CheckEvents(L);
 
-    int currwd, currht;
-    bigview->GetClientSize(&currwd, &currht);
-    if (currwd < 0) currwd = 0;
-    if (currht < 0) currht = 0;
+    int index = -1;
+    if (lua_gettop(L) > 0) {
+        index = luaL_checkinteger(L, 1);
+        if (index < 0 || index >= numlayers) {
+            char msg[64];
+            sprintf(msg, "getview error: bad index (%d)", index);
+            GollyError(L, msg);
+        }
+    }
+
+    int wd, ht;
+    if (index == -1) {
+        bigview->GetClientSize(&wd, &ht);
+    } else {
+        if (numlayers > 1 && tilelayers) {
+            // tilerect values are only valid when multiple layers are tiled
+            wd = GetLayer(index)->tilerect.width;
+            ht = GetLayer(index)->tilerect.height;
+        } else {
+            bigview->GetClientSize(&wd, &ht);
+        }
+    }
+    if (wd < 0) wd = 0;
+    if (ht < 0) ht = 0;
     
-    lua_pushinteger(L, currwd);
-    lua_pushinteger(L, currht);
+    lua_pushinteger(L, wd);
+    lua_pushinteger(L, ht);
     
-    return 2;   // result is 2 integers (wd, ht)
+    return 2;   // result is 2 integers (width and height)
 }
 
 // -----------------------------------------------------------------------------
@@ -1891,7 +1987,7 @@ static int g_autoupdate(lua_State* L)
 {
     CheckEvents(L);
 
-    autoupdate = lua_toboolean(L, 1) ? true : false;    // avoids stupid MVC warning
+    autoupdate = CheckBoolean(L, 1);
     
     return 0;   // no result
 }
@@ -2084,7 +2180,7 @@ static int g_getname(lua_State* L)
         GollyError(L, msg);
     }
     
-    lua_pushstring(L, (const char*)GetLayer(index)->currname.mb_str(wxConvUTF8));
+    lua_pushstring(L, (const char*)GetLayer(index)->currname.mb_str(LUA_ENC));
     
     return 1;   // result is a string
 }
@@ -2190,6 +2286,112 @@ static int g_getcolors(lua_State* L)
 
 // -----------------------------------------------------------------------------
 
+static int g_overlaytable(lua_State* L)
+{
+    CheckEvents(L);
+    
+    const char* result = NULL;
+    int nresults = 0;
+
+    // check the argument is a table
+    luaL_checktype(L, 1, LUA_TTABLE);
+    
+    // get the size of the table
+    int n = (int)lua_rawlen(L, 1);
+
+    // check if the table contains any elements
+    if (n > 0) {
+        // get the command name
+        lua_rawgeti(L, 1, 1);
+        const char* cmd = lua_tostring(L, -1);
+        lua_pop(L, 1);
+
+        // check the command name was a string
+        if (cmd) {
+            result = curroverlay->DoOverlayTable(cmd, L, n, &nresults);
+        } else {
+            result = "ERR:ovtable command name must be a string";
+        }
+    } else {
+        result = "ERR:missing ovtable command";
+    }
+
+    if (result == NULL) return nresults;   // no error so return any results
+    
+    if (result[0] == 'E' && result[1] == 'R' && result[2] == 'R' ) {
+        std::string msg = "ovtable error: ";
+        msg += result + 4;  // skip past "ERR:"
+        GollyError(L, msg.c_str());
+    }
+    
+    return 0;  // error so return nothing
+}
+
+// -----------------------------------------------------------------------------
+
+static int g_overlay(lua_State* L)
+{
+    CheckEvents(L);
+    
+    const char* cmd = luaL_checkstring(L, 1);
+    
+    const char* result = curroverlay->DoOverlayCommand(cmd);
+    
+    if (result == NULL) return 0;   // no error and no result
+    
+    if (result[0] == 'E' && result[1] == 'R' && result[2] == 'R' ) {
+        std::string msg = "overlay error: ";
+        msg += result + 4;  // skip past "ERR:"
+        GollyError(L, msg.c_str());
+    }
+    
+    lua_pushstring(L, result);
+    
+    return 1;   // result is a string
+}
+
+// -----------------------------------------------------------------------------
+
+static int g_os(lua_State* L)
+{
+    CheckEvents(L);
+
+    lua_pushstring(L, GSF_os());
+    
+    return 1;   // result is a string
+}
+
+// -----------------------------------------------------------------------------
+
+static int g_millisecs(lua_State* L)
+{
+    CheckEvents(L);
+
+    #if wxCHECK_VERSION(2,9,3)
+        wxLongLong t = stopwatch->TimeInMicro();
+        double d = t.ToDouble() / 1000.0L;
+        lua_pushnumber(L, (lua_Number) d);
+    #else
+        lua_pushnumber(L, (lua_Number) stopwatch->Time());
+    #endif
+    
+    return 1;   // result is a floating point number
+}
+
+// -----------------------------------------------------------------------------
+
+static int g_sleep(lua_State* L)
+{
+    CheckEvents(L);
+
+    int ms = luaL_checkinteger(L, 1);
+    wxMilliSleep(ms);
+    
+    return 0;   // no result
+}
+
+// -----------------------------------------------------------------------------
+
 static int g_setoption(lua_State* L)
 {
     CheckEvents(L);
@@ -2277,6 +2479,50 @@ static int g_getcolor(lua_State* L)
 
 // -----------------------------------------------------------------------------
 
+static int g_savechanges(lua_State* L)
+{
+    CheckEvents(L);
+    
+    wxString query = wxString(luaL_checkstring(L, 1), LUA_ENC);
+    wxString msg = wxString(luaL_checkstring(L, 2), LUA_ENC);
+
+    int answer = SaveChanges(query, msg);
+    switch (answer) {
+        case 2: {
+            // user selected Yes (or Save on Mac)
+            lua_pushstring(L, "yes");
+            break;
+        }
+        case 1: {
+            // user selected No (or Don't Save on Mac)
+            lua_pushstring(L, "no");
+            break;
+        }
+        default: {
+            // answer == 0 (user selected Cancel)
+            lua_pushstring(L, "cancel");
+        }
+    }
+    
+    return 1;   // result is a string
+}
+
+// -----------------------------------------------------------------------------
+
+static int g_settitle(lua_State* L)
+{
+    CheckEvents(L);
+    
+    // set scripttitle to avoid MainFrame::SetWindowTitle changing title
+    scripttitle = wxString(luaL_checkstring(L, 1), LUA_ENC);
+    
+    mainptr->SetTitle(scripttitle);
+    
+    return 0;   // no result
+}
+
+// -----------------------------------------------------------------------------
+
 static int g_setclipstr(lua_State* L)
 {
     CheckEvents(L);
@@ -2294,11 +2540,11 @@ static int g_getclipstr(lua_State* L)
 
     wxTextDataObject data;
     if (!mainptr->GetTextFromClipboard(&data)) {
-        GollyError(L, "getclipstr error: no text in clipboard.");
+        data.SetText(wxEmptyString);
     }
     
     wxString clipstr = data.GetText();
-    lua_pushstring(L, (const char*)clipstr.mb_str(wxConvUTF8));
+    lua_pushstring(L, (const char*)clipstr.mb_str(LUA_ENC));
     
     return 1;   // result is a string
 }
@@ -2323,7 +2569,7 @@ static int g_getstring(lua_State* L)
         lua_error(L);
     }
     
-    lua_pushstring(L, (const char*)result.mb_str(wxConvUTF8));
+    lua_pushstring(L, (const char*)result.mb_str(LUA_ENC));
     
     return 1;   // result is a string
 }
@@ -2337,7 +2583,7 @@ static int g_getxy(lua_State* L)
     statusptr->CheckMouseLocation(mainptr->infront);   // sets mousepos
     if (viewptr->showcontrols) mousepos = wxEmptyString;
     
-    lua_pushstring(L, (const char*)mousepos.mb_str(wxConvUTF8));
+    lua_pushstring(L, (const char*)mousepos.mb_str(LUA_ENC));
     
     return 1;   // result is a string
 }
@@ -2348,13 +2594,13 @@ static int g_getevent(lua_State* L)
 {
     CheckEvents(L);
 
-    int get = 1;
-    if (lua_gettop(L) > 0) get = lua_toboolean(L, 1) ? 1 : 0;
+    bool get = true;
+    if (lua_gettop(L) > 0) get = CheckBoolean(L, 1);
     
     wxString event;
-    GSF_getevent(event, get);
+    GSF_getevent(event, get ? 1 : 0);
     
-    lua_pushstring(L, (const char*)event.mb_str(wxConvUTF8));
+    lua_pushstring(L, (const char*)event.mb_str(LUA_ENC));
     
     return 1;   // result is a string
 }
@@ -2380,9 +2626,13 @@ static int g_doevent(lua_State* L)
 static int g_show(lua_State* L)
 {
     CheckEvents(L);
+    
+    // do NOT call luaL_checkstring when inscript is false
+    // (chaos will ensue if an error is detected)
+    const char* msg = luaL_checkstring(L, 1);
 
     inscript = false;
-    statusptr->DisplayMessage(wxString(luaL_checkstring(L, 1), LUA_ENC));
+    statusptr->DisplayMessage(wxString(msg, LUA_ENC));
     inscript = true;
     // make sure status bar is visible
     if (!showstatus) mainptr->ToggleStatusBar();
@@ -2396,8 +2646,12 @@ static int g_error(lua_State* L)
 {
     CheckEvents(L);
     
+    // do NOT call luaL_checkstring when inscript is false
+    // (chaos will ensue if an error is detected)
+    const char* msg = luaL_checkstring(L, 1);
+    
     inscript = false;
-    statusptr->ErrorMessage(wxString(luaL_checkstring(L, 1), LUA_ENC));
+    statusptr->ErrorMessage(wxString(msg, LUA_ENC));
     inscript = true;
     // make sure status bar is visible
     if (!showstatus) mainptr->ToggleStatusBar();
@@ -2410,8 +2664,12 @@ static int g_error(lua_State* L)
 static int g_warn(lua_State* L)
 {
     CheckEvents(L);
+    
+    const char* msg = luaL_checkstring(L, 1);
+    bool showCancel = true;
+    if (lua_gettop(L) > 1) showCancel = CheckBoolean(L, 2);
         
-    Warning(wxString(luaL_checkstring(L, 1), LUA_ENC));
+    Warning(wxString(msg, LUA_ENC), showCancel);
     
     return 0;   // no result
 }
@@ -2422,7 +2680,11 @@ static int g_note(lua_State* L)
 {
     CheckEvents(L);
     
-    Note(wxString(luaL_checkstring(L, 1), LUA_ENC));
+    const char* msg = luaL_checkstring(L, 1);
+    bool showCancel = true;
+    if (lua_gettop(L) > 1) showCancel = CheckBoolean(L, 2);
+    
+    Note(wxString(msg, LUA_ENC), showCancel);
     
     return 0;   // no result
 }
@@ -2445,11 +2707,11 @@ static int g_check(lua_State* L)
     // CheckEvents(L);
     // don't call CheckEvents here otherwise we can't safely write code like
     //    if g.getlayer() == target then
-    //       g.check(0)
+    //       g.check(false)
     //       ... do stuff to target layer ...
-    //       g.check(1)
+    //       g.check(true)
     
-    allowcheck = (luaL_checkinteger(L, 1) != 0);
+    allowcheck = CheckBoolean(L, 1);
     
     return 0;   // no result
 }
@@ -2499,6 +2761,8 @@ static const struct luaL_Reg gollyfuncs [] = {
     { "setdir",       g_setdir },       // set location of specified directory
     { "getdir",       g_getdir },       // return location of specified directory
     { "getfiles",     g_getfiles },     // return array of files in specified directory
+    { "getpath",      g_getpath },      // return the path to the current opened pattern
+    { "getinfo",      g_getinfo },      // return comments from pattern file
     // editing
     { "new",          g_new },          // create new universe and set window title
     { "cut",          g_cut },          // cut selection to clipboard
@@ -2514,6 +2778,7 @@ static const struct luaL_Reg gollyfuncs [] = {
     { "evolve",       g_evolve },       // generate pattern contained in given cell array
     { "putcells",     g_putcells },     // paste given cell array into current universe
     { "getcells",     g_getcells },     // return cell array in given rectangle
+    // { "getcells2",     g_getcells2 },     // experimental version (needs more thought!!!)
     { "join",         g_join },         // return concatenation of given cell arrays
     { "hash",         g_hash },         // return hash value for pattern in given rectangle
     { "getclip",      g_getclip },      // return pattern in clipboard (as wd, ht, cell array)
@@ -2571,7 +2836,14 @@ static const struct luaL_Reg gollyfuncs [] = {
     { "getname",      g_getname },      // get name of given layer
     { "setcolors",    g_setcolors },    // set color(s) used in current layer
     { "getcolors",    g_getcolors },    // get color(s) used in current layer
+    { "overlay",      g_overlay },      // do an overlay command from a string
+    { "ovtable",      g_overlaytable }, // do an overlay command from a table
     // miscellaneous
+    { "os",           g_os },           // return the current OS (Windows/Mac/Linux)
+    { "millisecs",    g_millisecs },    // return elapsed time since Golly started, in millisecs
+    { "sleep",        g_sleep },        // sleep for the given number of millisecs
+    { "savechanges",  g_savechanges },  // show standard save changes dialog and return answer
+    { "settitle",     g_settitle },     // set window title to given string
     { "setoption",    g_setoption },    // set given option to new value (and return old value)
     { "getoption",    g_getoption },    // return current value of given option
     { "setcolor",     g_setcolor },     // set given color to new r,g,b (returns old r,g,b)
@@ -2654,7 +2926,7 @@ void RunLuaScript(const wxString& filepath)
         // (definitely necessary on Windows because wxFILE_SEP_PATH is a backslash)
         pstring.Replace(wxT("\\"), wxT("\\\\"));
 
-        luaL_dostring(L, (const char*)pstring.mb_str(wxConvUTF8));
+        luaL_dostring(L, (const char*)pstring.mb_str(LUA_ENC));
     }
     
     lua_level++;
