@@ -1,26 +1,5 @@
-/*** /
- 
- This file is part of Golly, a Game of Life Simulator.
- Copyright (C) 2013 Andrew Trevorrow and Tomas Rokicki.
- 
- This program is free software; you can redistribute it and/or
- modify it under the terms of the GNU General Public License
- as published by the Free Software Foundation; either version 2
- of the License, or (at your option) any later version.
- 
- This program is distributed in the hope that it will be useful,
- but WITHOUT ANY WARRANTY; without even the implied warranty of
- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- GNU General Public License for more details.
- 
- You should have received a copy of the GNU General Public License
- along with this program; if not, write to the Free Software
- Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
- 
- Web site:  http://sourceforge.net/projects/golly
- Authors:   rokicki@gmail.com  andrew@trevorrow.com
- 
- / ***/
+// This file is part of Golly.
+// See docs/License.html for the copyright notice.
 
 #ifndef _WXMAIN_H_
 #define _WXMAIN_H_
@@ -59,6 +38,7 @@ public:
     bool CopyTextToClipboard(const wxString& text);
     bool GetTextFromClipboard(wxTextDataObject* data);
     bool ClipboardContainsRule();
+    bool ClipboardContainsRLE3();
     void OpenClipboard();
     void RunClipboard();
     
@@ -114,6 +94,7 @@ public:
     void ToggleAutoFit();
     void ToggleHyperspeed();
     void ToggleHashInfo();
+    void ToggleShowPopulation();
     void SetStepExponent(int newexpo);
     void SetGenIncrement();
     bool SaveStartingPattern();
@@ -135,6 +116,7 @@ public:
     void ToggleStatusBar();
     void ToggleExactNumbers();
     void ToggleToolBar();
+    void ToggleScrollBars();
     void ToggleFullScreen();
     void ShowPatternInfo();
     void ResizeSplitWindow(int wd, int ht);
@@ -143,6 +125,9 @@ public:
     wxWindow* RightPane();
     
     // layer functions
+    void SaveOverlay();
+    void ToggleOverlay();
+    void DeleteOverlay();
     void UpdateLayerItem(int index);
     void AppendLayerItem();
     void RemoveLayerItem();
@@ -173,11 +158,10 @@ public:
 
     bool infront;               // main window is active?
 
-#ifdef __WXMAC__
     // these scroll bars are needed to avoid bug in wxGLCanvas on Mac
+    // and to allow showing/hiding them on all platforms
     wxScrollBar* hbar;
     wxScrollBar* vbar;
-#endif
     
 private:
     // any class wishing to process wxWidgets events must use this macro
@@ -193,9 +177,7 @@ private:
     void OnSashDblClick(wxSplitterEvent& event);
     void OnGenTimer(wxTimerEvent& event);
     void OnClose(wxCloseEvent& event);
-#ifdef __WXMAC__
     void OnScroll(wxScrollEvent& event);
-#endif
     
     // file functions
     bool LoadImage(const wxString& path);
@@ -316,6 +298,7 @@ enum {
     ID_AUTO,
     ID_HYPER,
     ID_HINFO,
+    ID_SHOW_POP,
     ID_RECORD,
     ID_DELTIME,
     ID_SETALGO,
@@ -346,6 +329,7 @@ enum {
     ID_INVERT,
     ID_SMARTSCALE,
     ID_TIMELINE,
+    ID_SCROLL,
     ID_INFO,
     
     // Set Scale submenu
@@ -357,6 +341,9 @@ enum {
     ID_SCALE_32,
     
     // Layer menu
+    ID_SAVE_OVERLAY,
+    ID_SHOW_OVERLAY,
+    ID_DEL_OVERLAY,
     ID_ADD_LAYER,
     ID_CLONE,
     ID_DUPLICATE,
@@ -380,6 +367,7 @@ enum {
     ID_HELP_KEYBOARD,
     ID_HELP_MOUSE,
     ID_HELP_LUA,
+    ID_HELP_OVERLAY,
     ID_HELP_PYTHON,
     ID_HELP_LEXICON,
     ID_HELP_ARCHIVES,
