@@ -1,26 +1,5 @@
-/*** /
- 
- This file is part of Golly, a Game of Life Simulator.
- Copyright (C) 2013 Andrew Trevorrow and Tomas Rokicki.
- 
- This program is free software; you can redistribute it and/or
- modify it under the terms of the GNU General Public License
- as published by the Free Software Foundation; either version 2
- of the License, or (at your option) any later version.
- 
- This program is distributed in the hope that it will be useful,
- but WITHOUT ANY WARRANTY; without even the implied warranty of
- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- GNU General Public License for more details.
- 
- You should have received a copy of the GNU General Public License
- along with this program; if not, write to the Free Software
- Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
- 
- Web site:  http://sourceforge.net/projects/golly
- Authors:   rokicki@gmail.com  andrew@trevorrow.com
- 
- / ***/
+// This file is part of Golly.
+// See docs/License.html for the copyright notice.
 
 #include "wx/wxprec.h"     // for compilers that support precompilation
 #ifndef WX_PRECOMP
@@ -301,7 +280,16 @@ void AlgoHelp::DisplayFile(const wxString& filepath)
         SetPage(contents);
         
     } else {
+        #if defined(__WXMSW__) && wxCHECK_VERSION(3,1,0)
+        if (filepath.StartsWith("file:")) {
+            LoadFile(filepath.Mid(8));
+        }
+        else {
+            LoadFile(filepath);
+        }
+        #else
         LoadPage(filepath);
+        #endif
     }
 }
 
@@ -696,11 +684,14 @@ void RuleDialog::UpdateHelp()
     if (expanded) {
         if (algoindex < NumAlgos()) {
             // display Help/Algorithms/algoname.html
+            wxString filename = wxString(GetAlgoName(algoindex),wxConvLocal);
+            // replace any spaces with underscores
+            filename.Replace(wxT(" "),wxT("_"));
             wxString filepath = gollydir + wxT("Help");
             filepath += wxFILE_SEP_PATH;
             filepath += wxT("Algorithms");
             filepath += wxFILE_SEP_PATH;
-            filepath += wxString(GetAlgoName(algoindex),wxConvLocal);
+            filepath += filename;
             filepath += wxT(".html");
             htmlwin->DisplayFile(filepath);
         } else {
@@ -798,6 +789,9 @@ void RuleDialog::OnChooseAlgo(wxCommandEvent& event)
                 }
                 defrule += wxT(":");
                 defrule += thisrule.AfterFirst(':');
+                // check that suffix is valid in this algo (if not, use default rule)
+                err = tempalgo->setrule( defrule.mb_str(wxConvLocal) );
+                if (err) defrule = wxString(tempalgo->DefaultRule(), wxConvLocal);
             }
             ignore_text_change = true;
             ruletext->SetValue(defrule);
