@@ -1,26 +1,7 @@
-/*** /
- 
- This file is part of Golly, a Game of Life Simulator.
- Copyright (C) 2013 Andrew Trevorrow and Tomas Rokicki.
- 
- This program is free software; you can redistribute it and/or
- modify it under the terms of the GNU General Public License
- as published by the Free Software Foundation; either version 2
- of the License, or (at your option) any later version.
- 
- This program is distributed in the hope that it will be useful,
- but WITHOUT ANY WARRANTY; without even the implied warranty of
- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- GNU General Public License for more details.
- 
- You should have received a copy of the GNU General Public License
- along with this program; if not, write to the Free Software
- Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
- 
- Web site:  http://sourceforge.net/projects/golly
- Authors:   rokicki@gmail.com  andrew@trevorrow.com
- 
- / ***/
+// This file is part of Golly.
+// See docs/License.html for the copyright notice.
+
+#include <string.h>        // for strlen and strcpy
 
 #include "wx/wxprec.h"     // for compilers that support precompilation
 #ifndef WX_PRECOMP
@@ -41,24 +22,29 @@
 #include "wxundo.h"        // for undoredo->...
 #include "wxalgos.h"       // for *_ALGO, algoinfo
 #include "wxlayer.h"       // for currlayer, SyncClones
-#include "wxtimeline.h"    // for TimelineExists
+#include "wxtimeline.h"    // for TimelineExists, ToggleTimelineBar
 #include "wxlua.h"         // for RunLuaScript, AbortLuaScript
 #include "wxperl.h"        // for RunPerlScript, AbortPerlScript
 #include "wxpython.h"      // for RunPythonScript, AbortPythonScript
+#include "wxoverlay.h"     // for curroverlay
 #include "wxscript.h"
 
 // =============================================================================
 
 // exported globals:
 bool inscript = false;     // a script is running?
-bool passkeys;             // pass keyboard events to script?
-bool passclicks;           // pass mouse events to script?
+bool pass_key_events;      // pass keyboard events to script?
+bool pass_mouse_events;    // pass mouse events to script?
+bool pass_file_events;     // pass file events to script?
 bool canswitch;            // can user switch layers while script is running?
 bool stop_after_script;    // stop generating pattern after running script?
 bool autoupdate;           // update display after each change to current universe?
 bool allowcheck;           // allow event checking?
-wxString scripterr;        // Perl/Python error message
+bool showprogress;         // script can display the progress dialog?
+wxString scripterr;        // Lua/Perl/Python error message
 wxString mousepos;         // current mouse position
+wxString scripttitle;      // window title set by settitle command
+wxString rle3path;         // path of .rle3 file to be sent to 3D.lua via GSF_getevent
 
 // local globals:
 static bool luascript = false;      // a Lua script is running?
@@ -69,7 +55,10 @@ static bool updateedit;             // need to update edit bar?
 static bool exitcalled;             // GSF_exit was called?
 static wxString scriptchars;        // non-escape chars saved by PassKeyToScript
 static wxString scriptloc;          // location of script file
-static wxArrayString eventqueue;    // FIFO queue for keyboard/mouse events 
+static wxArrayString eventqueue;    // FIFO queue for keyboard/mouse events
+
+// constants:
+const int maxcomments = 128 * 1024; // maximum comment size
 
 // -----------------------------------------------------------------------------
 
@@ -83,6 +72,13 @@ void DoAutoUpdate()
             showtitle = false;
         }
         inscript = true;
+
+        #ifdef __WXGTK__
+            // needed on Linux to see update immediately
+            insideYield = true;
+            wxGetApp().Yield(true);
+            insideYield = false;
+        #endif
     }
 }
 
@@ -130,10 +126,18 @@ const char* GSF_open(const wxString& filename, int remember)
         return "open error: given file does not exist.";
     }
     
+    // temporarily set pass_file_events to false so OpenFile won't pass
+    // a file event back to this script
+    bool savepass = pass_file_events;
+    pass_file_events = false;
+    
     // only add file to Open Recent submenu if remember flag is non-zero
     mainptr->OpenFile(fullpath, remember != 0);
     DoAutoUpdate();
     
+    // restore pass_file_events
+    pass_file_events = savepass;
+    
     return NULL;
 }
 
@@ -220,6 +224,22 @@ const char* GSF_getdir(const char* dirname)
 
 // -----------------------------------------------------------------------------
 
+const char* GSF_os()
+{
+    // return a string that specifies the current operating system
+    #if defined(__WXMSW__)
+        return "Windows";
+    #elif defined(__WXMAC__)
+        return "Mac";
+    #elif defined(__WXGTK__)
+        return "Linux";
+    #else 
+        return "unknown";
+    #endif
+}
+
+// -----------------------------------------------------------------------------
+
 const char* GSF_setalgo(const char* algostring)
 {
     // find index for given algo name
@@ -692,6 +712,19 @@ bool GSF_setoption(const char* optname, int newval, int* oldval)
             DoAutoUpdate();
         }
         
+    } else if (strcmp(optname, "showbuttons") == 0) {
+        *oldval = controlspos;
+        if (newval < 0) newval = 0;
+        if (newval > 4) newval = 4;
+        if (*oldval != newval) {
+            // update position of translucent buttons
+            controlspos = newval;
+            int wd, ht;
+            viewptr->GetClientSize(&wd, &ht);
+            viewptr->SetViewSize(wd, ht);
+            DoAutoUpdate();
+        }
+        
     } else if (strcmp(optname, "showeditbar") == 0) {
         *oldval = showedit ? 1 : 0;
         if (*oldval != newval) {
@@ -718,6 +751,13 @@ bool GSF_setoption(const char* optname, int newval, int* oldval)
         if (*oldval != newval)
             mainptr->ToggleHashInfo();
         
+    } else if (strcmp(optname, "showpopulation") == 0) {
+        *oldval = showpopulation ? 1 : 0;
+        if (*oldval != newval) {
+            mainptr->ToggleShowPopulation();
+            DoAutoUpdate();
+        }
+        
     } else if (strcmp(optname, "showicons") == 0) {
         *oldval = showicons ? 1 : 0;
         if (*oldval != newval) {
@@ -732,6 +772,20 @@ bool GSF_setoption(const char* optname, int newval, int* oldval)
             DoAutoUpdate();
         }
         
+    } else if (strcmp(optname, "showoverlay") == 0) {
+        *oldval = showoverlay ? 1 : 0;
+        if (*oldval != newval) {
+            showoverlay = !showoverlay;
+            DoAutoUpdate();
+        }
+        
+    } else if (strcmp(optname, "showprogress") == 0) {
+        *oldval = showprogress ? 1 : 0;
+        if (*oldval != newval) {
+            showprogress = !showprogress;
+            // no need for DoAutoUpdate();
+        }
+        
     } else if (strcmp(optname, "showfiles") == 0 ||
                strcmp(optname, "showpatterns") == 0) {      // deprecated
         *oldval = showfiles ? 1 : 0;
@@ -747,6 +801,13 @@ bool GSF_setoption(const char* optname, int newval, int* oldval)
             DoAutoUpdate();
         }
         
+    } else if (strcmp(optname, "showscrollbars") == 0) {
+        *oldval = showscrollbars ? 1 : 0;
+        if (*oldval != newval) {
+            mainptr->ToggleScrollBars();
+            DoAutoUpdate();
+        }
+        
     } else if (strcmp(optname, "showstatusbar") == 0) {
         *oldval = showstatus ? 1 : 0;
         if (*oldval != newval) {
@@ -754,6 +815,13 @@ bool GSF_setoption(const char* optname, int newval, int* oldval)
             DoAutoUpdate();
         }
         
+    } else if (strcmp(optname, "showtimeline") == 0) {
+        *oldval = showtimeline ? 1 : 0;
+        if (*oldval != newval) {
+            ToggleTimelineBar();
+            DoAutoUpdate();
+        }
+        
     } else if (strcmp(optname, "showtoolbar") == 0) {
         *oldval = showtool ? 1 : 0;
         if (*oldval != newval) {
@@ -834,36 +902,42 @@ bool GSF_setoption(const char* optname, int newval, int* oldval)
 
 bool GSF_getoption(const char* optname, int* optval)
 {
-    if      (strcmp(optname, "autofit") == 0)       *optval = currlayer->autofit ? 1 : 0;
-    else if (strcmp(optname, "boldspacing") == 0)   *optval = boldspacing;
-    else if (strcmp(optname, "drawingstate") == 0)  *optval = currlayer->drawingstate;
-    else if (strcmp(optname, "fullscreen") == 0)    *optval = mainptr->fullscreen ? 1 : 0;
-    else if (strcmp(optname, "hyperspeed") == 0)    *optval = currlayer->hyperspeed ? 1 : 0;
-    else if (strcmp(optname, "mindelay") == 0)      *optval = mindelay;
-    else if (strcmp(optname, "maxdelay") == 0)      *optval = maxdelay;
-    else if (strcmp(optname, "opacity") == 0)       *optval = opacity;
-    else if (strcmp(optname, "restoreview") == 0)   *optval = restoreview ? 1 : 0;
-    else if (strcmp(optname, "savexrle") == 0)      *optval = savexrle ? 1 : 0;
-    else if (strcmp(optname, "showallstates") == 0) *optval = showallstates ? 1 : 0;
-    else if (strcmp(optname, "showboldlines") == 0) *optval = showboldlines ? 1 : 0;
-    else if (strcmp(optname, "showeditbar") == 0)   *optval = showedit ? 1 : 0;
-    else if (strcmp(optname, "showexact") == 0)     *optval = showexact ? 1 : 0;
-    else if (strcmp(optname, "showgrid") == 0)      *optval = showgridlines ? 1 : 0;
-    else if (strcmp(optname, "showhashinfo") == 0)  *optval = currlayer->showhashinfo ? 1 : 0;
-    else if (strcmp(optname, "showicons") == 0)     *optval = showicons ? 1 : 0;
-    else if (strcmp(optname, "showlayerbar") == 0)  *optval = showlayer ? 1 : 0;
-    else if (strcmp(optname, "showfiles") == 0)     *optval = showfiles ? 1 : 0;
-    else if (strcmp(optname, "showpatterns") == 0)  *optval = showfiles ? 1 : 0;    // deprecated
-    else if (strcmp(optname, "showscripts") == 0)   *optval = 0;                    // ditto
-    else if (strcmp(optname, "showstatusbar") == 0) *optval = showstatus ? 1 : 0;
-    else if (strcmp(optname, "showtoolbar") == 0)   *optval = showtool ? 1 : 0;
-    else if (strcmp(optname, "smartscale") == 0)    *optval = smartscale ? 1 : 0;
-    else if (strcmp(optname, "stacklayers") == 0)   *optval = stacklayers ? 1 : 0;
-    else if (strcmp(optname, "swapcolors") == 0)    *optval = swapcolors ? 1 : 0;
-    else if (strcmp(optname, "switchlayers") == 0)  *optval = canswitch ? 1 : 0;
-    else if (strcmp(optname, "synccursors") == 0)   *optval = synccursors ? 1 : 0;
-    else if (strcmp(optname, "syncviews") == 0)     *optval = syncviews ? 1 : 0;
-    else if (strcmp(optname, "tilelayers") == 0)    *optval = tilelayers ? 1 : 0;
+    if      (strcmp(optname, "autofit") == 0)           *optval = currlayer->autofit ? 1 : 0;
+    else if (strcmp(optname, "boldspacing") == 0)       *optval = boldspacing;
+    else if (strcmp(optname, "drawingstate") == 0)      *optval = currlayer->drawingstate;
+    else if (strcmp(optname, "fullscreen") == 0)        *optval = mainptr->fullscreen ? 1 : 0;
+    else if (strcmp(optname, "hyperspeed") == 0)        *optval = currlayer->hyperspeed ? 1 : 0;
+    else if (strcmp(optname, "mindelay") == 0)          *optval = mindelay;
+    else if (strcmp(optname, "maxdelay") == 0)          *optval = maxdelay;
+    else if (strcmp(optname, "opacity") == 0)           *optval = opacity;
+    else if (strcmp(optname, "restoreview") == 0)       *optval = restoreview ? 1 : 0;
+    else if (strcmp(optname, "savexrle") == 0)          *optval = savexrle ? 1 : 0;
+    else if (strcmp(optname, "showallstates") == 0)     *optval = showallstates ? 1 : 0;
+    else if (strcmp(optname, "showboldlines") == 0)     *optval = showboldlines ? 1 : 0;
+    else if (strcmp(optname, "showbuttons") == 0)       *optval = controlspos;
+    else if (strcmp(optname, "showeditbar") == 0)       *optval = showedit ? 1 : 0;
+    else if (strcmp(optname, "showexact") == 0)         *optval = showexact ? 1 : 0;
+    else if (strcmp(optname, "showgrid") == 0)          *optval = showgridlines ? 1 : 0;
+    else if (strcmp(optname, "showhashinfo") == 0)      *optval = currlayer->showhashinfo ? 1 : 0;
+    else if (strcmp(optname, "showpopulation") == 0)    *optval = showpopulation ? 1 : 0;
+    else if (strcmp(optname, "showicons") == 0)         *optval = showicons ? 1 : 0;
+    else if (strcmp(optname, "showlayerbar") == 0)      *optval = showlayer ? 1 : 0;
+    else if (strcmp(optname, "showoverlay") == 0)       *optval = showoverlay ? 1 : 0;
+    else if (strcmp(optname, "showprogress") == 0)      *optval = showprogress ? 1 : 0;
+    else if (strcmp(optname, "showfiles") == 0)         *optval = showfiles ? 1 : 0;
+    else if (strcmp(optname, "showpatterns") == 0)      *optval = showfiles ? 1 : 0;        // deprecated
+    else if (strcmp(optname, "showscripts") == 0)       *optval = 0;                        // ditto
+    else if (strcmp(optname, "showscrollbars") == 0)    *optval = showscrollbars ? 1 : 0;
+    else if (strcmp(optname, "showstatusbar") == 0)     *optval = showstatus ? 1 : 0;
+    else if (strcmp(optname, "showtimeline") == 0)      *optval = showtimeline ? 1 : 0;
+    else if (strcmp(optname, "showtoolbar") == 0)       *optval = showtool ? 1 : 0;
+    else if (strcmp(optname, "smartscale") == 0)        *optval = smartscale ? 1 : 0;
+    else if (strcmp(optname, "stacklayers") == 0)       *optval = stacklayers ? 1 : 0;
+    else if (strcmp(optname, "swapcolors") == 0)        *optval = swapcolors ? 1 : 0;
+    else if (strcmp(optname, "switchlayers") == 0)      *optval = canswitch ? 1 : 0;
+    else if (strcmp(optname, "synccursors") == 0)       *optval = synccursors ? 1 : 0;
+    else if (strcmp(optname, "syncviews") == 0)         *optval = syncviews ? 1 : 0;
+    else if (strcmp(optname, "tilelayers") == 0)        *optval = tilelayers ? 1 : 0;
     // this option is deprecated (use getalgo command)
     else if (strcmp(optname, "hashing") == 0)
         *optval = (currlayer->algtype == HLIFE_ALGO) ? 1 : 0;
@@ -997,12 +1071,22 @@ bool GSF_getcolor(const char* colname, wxColor& color)
 void GSF_getevent(wxString& event, int get)
 {
     if (get) {
-        passkeys = true;     // future keyboard events will call PassKeyToScript
-        passclicks = true;   // future mouse events will call PassClickToScript
+        pass_key_events = true;     // future keyboard events will call PassKeyToScript
+        pass_mouse_events = true;   // future mouse events will call PassClickToScript
+        pass_file_events = true;    // future open file evenst will call PassFileToScript
+        
+        // rle3path is non-empty if Golly has just seen a .rle3 file and started up 3D.lua
+        if (rle3path[0]) {
+            event = wxT("file ") + rle3path;
+            rle3path = wxEmptyString;
+            return;
+        }
+        
     } else {
-        // tell Golly to handle future keyboard and mouse events
-        passkeys = false;
-        passclicks = false;
+        // tell Golly to handle future keyboard/mouse/file events
+        pass_key_events = false;
+        pass_mouse_events = false;
+        pass_file_events = false;
         // clear any pending events so event is set to empty string below
         eventqueue.Clear();
     }
@@ -1081,7 +1165,7 @@ const char* GSF_doevent(const wxString& event)
                 }
             } else if (event[5] != ' ') {
                 // parse special char name like space, tab, etc
-                // must match reverse conversion in PassKeyToScript
+                // must match reverse conversion in PassKeyToScript and PassKeyUpToScript
                 if (event.Contains(wxT("space")))      key = ' '; else
                 if (event.Contains(wxT("home")))       key = WXK_HOME; else
                 if (event.Contains(wxT("end")))        key = WXK_END; else
@@ -1110,6 +1194,33 @@ const char* GSF_doevent(const wxString& event)
                 showtitle = false;
             }
             
+        } else if (event.StartsWith(wxT("zoom"))) {
+            // parse event string like "zoomin 10 20" or "zoomout 10 20"
+            wxString xstr = event.AfterFirst(' ');
+            wxString ystr = xstr.AfterFirst(' ');
+            xstr = xstr.BeforeFirst(' ');
+            if (!xstr.IsNumber()) return "Bad x value.";
+            if (!ystr.IsNumber()) return "Bad y value.";
+            int x = wxAtoi(xstr);
+            int y = wxAtoi(ystr);
+
+            // x,y is pixel position in viewport
+            if (event.StartsWith(wxT("zoomin"))) {
+                viewptr->TestAutoFit();
+                if (currlayer->view->getmag() < MAX_MAG) {
+                    currlayer->view->zoom(x, y);
+                }
+            } else {
+                viewptr->TestAutoFit();
+                currlayer->view->unzoom(x, y);
+            }
+
+            inscript = false;
+            mainptr->UpdatePatternAndStatus();
+            bigview->UpdateScrollBars();
+            inscript = true;
+            mainptr->UpdateUserInterface();
+            
         } else if (event.StartsWith(wxT("click "))) {
             // parse event string like "click 10 20 left altshift"
             wxString xstr = event.AfterFirst(' ');
@@ -1145,7 +1256,23 @@ const char* GSF_doevent(const wxString& event)
                 // ignore click if x,y is outside viewport or grid
                 return NULL;
             }
-            
+
+        } else if (event.StartsWith(wxT("kup "))) {
+            // ignore key up event
+            return NULL;
+
+        } else if (event.StartsWith(wxT("mup "))) {
+            // ignore mouse up event
+            return NULL;
+
+        } else if (event.StartsWith(wxT("file "))) {
+            // ignore file event (scripts can call GSF_open)
+            return NULL;
+
+        } else if (event.StartsWith(wxT("o"))) {
+            // ignore oclick/ozoomin/ozoomout event in overlay
+            return NULL;
+
         } else {
             return "Unknown event.";
         }
@@ -1159,7 +1286,7 @@ const char* GSF_doevent(const wxString& event)
 
 char GSF_getkey()
 {
-    passkeys = true;   // future keyboard events will call PassKeyToScript
+    pass_key_events = true;   // future keyboard events will call PassKeyToScript
     
     if (scriptchars.length() == 0) {
         // return empty string
@@ -1229,6 +1356,13 @@ void GSF_update()
     }
     
     inscript = true;
+
+    #ifdef __WXGTK__
+        // needed on Linux to see update immediately
+        insideYield = true;
+        wxGetApp().Yield(true);
+        insideYield = false;
+    #endif
 }
 
 // -----------------------------------------------------------------------------
@@ -1247,6 +1381,47 @@ void GSF_exit(const wxString& errmsg)
     exitcalled = true;   // prevent CheckScriptError changing message
 }
 
+#ifdef __WXMAC__
+    // convert path to decomposed UTF8 so fopen will work
+    #define CURRFILE currlayer->currfile.fn_str()
+#else
+    #define CURRFILE currlayer->currfile.mb_str(wxConvLocal)
+#endif
+
+// -----------------------------------------------------------------------------
+
+const char* GSF_getpath()
+{
+    // need to be careful converting Unicode wxString to char*
+    static wxCharBuffer path;
+    path = CURRFILE;
+    return (const char*) path;
+}
+
+// -----------------------------------------------------------------------------
+
+const char* GSF_getinfo()
+{
+    // comment buffer
+    static char comments[maxcomments];
+
+    // buffer for receiving comment data (allocated by readcomments)
+    char *commptr = NULL;
+
+    // read the comments in the pattern file
+    const char* err = readcomments(CURRFILE, &commptr);
+    if (err) {
+        free(commptr);
+        return "";
+    }
+
+    // copy the comments and truncate to buffer size if longer
+    strncpy(comments, commptr, maxcomments);
+    comments[maxcomments - 1] = '\0';
+    free(commptr);
+    return comments;
+}
+
 // =============================================================================
 
 void CheckScriptError(const wxString& ext)
@@ -1346,6 +1521,7 @@ void RunScript(const wxString& filename)
     } else {
         mainptr->showbanner = false;
         statusptr->ClearMessage();
+        scripttitle.Clear();
         scripterr.Clear();
         scriptchars.Clear();
         eventqueue.Clear();
@@ -1354,10 +1530,12 @@ void RunScript(const wxString& filename)
         autoupdate = false;
         exitcalled = false;
         allowcheck = true;
+        showprogress = true;
         showtitle = false;
         updateedit = false;
-        passkeys = false;
-        passclicks = false;
+        pass_key_events = false;
+        pass_mouse_events = false;
+        pass_file_events = false;
         wxGetApp().PollerReset();
     }
     
@@ -1497,6 +1675,11 @@ void RunScript(const wxString& filename)
         // display any error message
         CheckScriptError(ext);
         
+        if (!scripttitle.IsEmpty()) {
+            scripttitle.Clear();
+            showtitle = true;
+        }
+        
         // update title, menu bar, cursor, viewport, status bar, tool bar, etc
         if (showtitle) mainptr->SetWindowTitle(wxEmptyString);
         mainptr->UpdateEverything();
@@ -1508,6 +1691,21 @@ void RunScript(const wxString& filename)
 
 // -----------------------------------------------------------------------------
 
+void PassOverlayClickToScript(int ox, int oy, int button, int modifiers)
+{
+    // build a string like "oclick 30 50 left none" and add to event queue
+    // for possible consumption by GSF_getevent
+    wxString clickinfo;
+    clickinfo.Printf(wxT("oclick %d %d"), ox, oy);
+    if (button == wxMOUSE_BTN_LEFT)     clickinfo += wxT(" left ");
+    if (button == wxMOUSE_BTN_MIDDLE)   clickinfo += wxT(" middle ");
+    if (button == wxMOUSE_BTN_RIGHT)    clickinfo += wxT(" right ");
+    AppendModifiers(modifiers, clickinfo);
+    eventqueue.Add(clickinfo);
+}
+
+// -----------------------------------------------------------------------------
+
 void PassClickToScript(const bigint& x, const bigint& y, int button, int modifiers)
 {
     // build a string like "click 10 20 left altshift" and add to event queue
@@ -1525,6 +1723,105 @@ void PassClickToScript(const bigint& x, const bigint& y, int button, int modifie
 
 // -----------------------------------------------------------------------------
 
+void PassMouseUpToScript(int button)
+{
+    // build a string like "mup left" and add to event queue
+    // for possible consumption by GSF_getevent
+    wxString minfo = wxT("mup ");
+    if (button == wxMOUSE_BTN_LEFT)     minfo += wxT("left");
+    if (button == wxMOUSE_BTN_MIDDLE)   minfo += wxT("middle");
+    if (button == wxMOUSE_BTN_RIGHT)    minfo += wxT("right");
+    eventqueue.Add(minfo);
+}
+
+// -----------------------------------------------------------------------------
+
+void PassZoomInToScript(int x, int y)
+{
+    int ox, oy;
+    if (showoverlay && curroverlay->PointInOverlay(x, y, &ox, &oy)
+                    && !curroverlay->TransparentPixel(ox, oy)) {
+        // zoom in to the overlay pixel at ox,oy
+        wxString zinfo;
+        zinfo.Printf(wxT("ozoomin %d %d"), ox, oy);
+        eventqueue.Add(zinfo);
+    
+    } else {
+        // zoom in to the viewport pixel at x,y (note that it's best not to
+        // pass the corresponding cell position because a doevent call will result
+        // in unwanted drifting due to conversion back to a pixel position)
+        wxString zinfo;
+        zinfo.Printf(wxT("zoomin %d %d"), x, y);
+        eventqueue.Add(zinfo);
+    }
+}
+
+// -----------------------------------------------------------------------------
+
+void PassZoomOutToScript(int x, int y)
+{
+    int ox, oy;
+    if (showoverlay && curroverlay->PointInOverlay(x, y, &ox, &oy)
+                    && !curroverlay->TransparentPixel(ox, oy)) {
+        // zoom out from the overlay pixel at ox,oy
+        wxString zinfo;
+        zinfo.Printf(wxT("ozoomout %d %d"), ox, oy);
+        eventqueue.Add(zinfo);
+    
+    } else {
+        // zoom out from the viewport pixel at x,y
+        wxString zinfo;
+        zinfo.Printf(wxT("zoomout %d %d"), x, y);
+        eventqueue.Add(zinfo);
+    }
+}
+
+// -----------------------------------------------------------------------------
+
+void PassKeyUpToScript(int key)
+{
+    // build a string like "kup x" and add to event queue
+    // for possible consumption by GSF_getevent
+    wxString keyinfo = wxT("kup ");
+    if (key > ' ' && key <= '~') {
+        // displayable ASCII
+        if (key >= 'A' && key <= 'Z') key += 32;  // convert A..Z to a..z to match case in key event
+        keyinfo += wxChar(key);
+    } else if (key >= WXK_F1 && key <= WXK_F24) {
+        // function key
+        keyinfo += wxString::Format(wxT("f%d"), key - WXK_F1 + 1);
+    } else {
+        // convert some special key codes to names like space, tab, delete, etc
+        // (must match reverse conversion in GSF_doevent)
+        switch (key) {
+            case ' ':               keyinfo += wxT("space");      break;
+            case WXK_HOME:          keyinfo += wxT("home");       break;
+            case WXK_END:           keyinfo += wxT("end");        break;
+            case WXK_PAGEUP:        keyinfo += wxT("pageup");     break;
+            case WXK_PAGEDOWN:      keyinfo += wxT("pagedown");   break;
+            case WXK_HELP:          keyinfo += wxT("help");       break;
+            case WXK_INSERT:        keyinfo += wxT("insert");     break;
+            case WXK_BACK:          // treat backspace like delete
+            case WXK_DELETE:        keyinfo += wxT("delete");     break;
+            case WXK_TAB:           keyinfo += wxT("tab");        break;
+            case WXK_NUMPAD_ENTER:  // treat enter like return
+            case WXK_RETURN:        keyinfo += wxT("return");     break;
+            case WXK_LEFT:          keyinfo += wxT("left");       break;
+            case WXK_RIGHT:         keyinfo += wxT("right");      break;
+            case WXK_UP:            keyinfo += wxT("up");         break;
+            case WXK_DOWN:          keyinfo += wxT("down");       break;
+            case WXK_ADD:           keyinfo += wxT("+");          break;
+            case WXK_SUBTRACT:      keyinfo += wxT("-");          break;
+            case WXK_DIVIDE:        keyinfo += wxT("/");          break;
+            case WXK_MULTIPLY:      keyinfo += wxT("*");          break;
+            default:                return;  // ignore all other key codes
+        }
+    }
+    eventqueue.Add(keyinfo);
+}
+
+// -----------------------------------------------------------------------------
+
 void PassKeyToScript(int key, int modifiers)
 {
     if (key == WXK_ESCAPE) {
@@ -1614,6 +1911,15 @@ void PassKeyToScript(int key, int modifiers)
 
 // -----------------------------------------------------------------------------
 
+void PassFileToScript(const wxString& filepath)
+{
+    wxString fileinfo = wxT("file ");
+    fileinfo += filepath;
+    eventqueue.Add(fileinfo);
+}
+
+// -----------------------------------------------------------------------------
+
 void FinishScripting()
 {
     // called when main window is closing (ie. app is quitting)
