@@ -1,26 +1,5 @@
-/*** /
- 
- This file is part of Golly, a Game of Life Simulator.
- Copyright (C) 2013 Andrew Trevorrow and Tomas Rokicki.
- 
- This program is free software; you can redistribute it and/or
- modify it under the terms of the GNU General Public License
- as published by the Free Software Foundation; either version 2
- of the License, or (at your option) any later version.
- 
- This program is distributed in the hope that it will be useful,
- but WITHOUT ANY WARRANTY; without even the implied warranty of
- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- GNU General Public License for more details.
- 
- You should have received a copy of the GNU General Public License
- along with this program; if not, write to the Free Software
- Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
- 
- Web site:  http://sourceforge.net/projects/golly
- Authors:   rokicki@gmail.com  andrew@trevorrow.com
- 
- / ***/
+// This file is part of Golly.
+// See docs/License.html for the copyright notice.
 
 #include "wx/wxprec.h"     // for compilers that support precompilation
 #ifndef WX_PRECOMP
@@ -282,7 +261,7 @@ void Selection::Advance()
     bool savecells = allowundo && !currlayer->stayclean;
     if (savecells && inscript) SavePendingChanges();
     
-    bool boundedgrid = (currlayer->algo->gridwd > 0 || currlayer->algo->gridht > 0);
+    bool boundedgrid = currlayer->algo->unbounded && (currlayer->algo->gridwd > 0 || currlayer->algo->gridht > 0);
     
     // check if selection encloses entire pattern;
     // can't do this if qlife because it uses gen parity to decide which bits to draw;
@@ -420,7 +399,7 @@ void Selection::AdvanceOutside()
     bool savecells = allowundo && !currlayer->stayclean;
     if (savecells && inscript) SavePendingChanges();
     
-    bool boundedgrid = (currlayer->algo->gridwd > 0 || currlayer->algo->gridht > 0);
+    bool boundedgrid = currlayer->algo->unbounded && (currlayer->algo->gridwd > 0 || currlayer->algo->gridht > 0);
     
     // check if selection is completely outside pattern edges;
     // can't do this if qlife because it uses gen parity to decide which bits to draw;
@@ -790,7 +769,7 @@ void Selection::Fit()
     int mag = MAX_MAG;
     while (true) {
         currlayer->view->setpositionmag(newx, newy, mag);
-        if ( currlayer->view->contains(selleft, seltop) &&
+        if (currlayer->view->contains(selleft, seltop) &&
             currlayer->view->contains(selright, selbottom) )
             break;
         mag--;
@@ -1537,7 +1516,7 @@ void Selection::RandomFill()
     BeginProgress(_("Randomly filling selection"));
     int cx, cy;
     lifealgo* curralgo = currlayer->algo;
-    int livestates = curralgo->NumCellStates() - 1;    // don't count dead state
+    int livestates = curralgo->NumRandomizedCellStates() - 1;    // don't count dead state
     
     for ( cy=itop; cy<=ibottom; cy++ ) {
         for ( cx=ileft; cx<=iright; cx++ ) {
@@ -1970,12 +1949,12 @@ bool Selection::Rotate(bool clockwise, bool inundoredo)
     if (!inundoredo) {
         // check if rotated selection edges are outside bounded grid
         if ( (currlayer->algo->gridwd > 0 &&
-              (newleft < currlayer->algo->gridleft || newright > currlayer->algo->gridright)) ||
-            (currlayer->algo->gridht > 0 &&
-             (newtop < currlayer->algo->gridtop || newbottom > currlayer->algo->gridbottom)) ) {
-                statusptr->ErrorMessage(_("New selection would be outside grid boundary."));
-                return false;
-            }
+                (newleft < currlayer->algo->gridleft || newright > currlayer->algo->gridright)) ||
+             (currlayer->algo->gridht > 0 &&
+                (newtop < currlayer->algo->gridtop || newbottom > currlayer->algo->gridbottom)) ) {
+            statusptr->ErrorMessage(_("New selection would be outside grid boundary."));
+            return false;
+        }
     }
     
     // if there is no pattern then just rotate the selection edges
@@ -1996,7 +1975,7 @@ bool Selection::Rotate(bool clockwise, bool inundoredo)
     bigint top, left, bottom, right;
     currlayer->algo->findedges(&top, &left, &bottom, &right);
     if ( (seltop > bottom || selbottom < top || selleft > right || selright < left) &&
-        (newtop > bottom || newbottom < top || newleft > right || newright < left) ) {
+         (newtop > bottom || newbottom < top || newleft > right || newright < left) ) {
         viewptr->SaveCurrentSelection();
         seltop    = newtop;
         selbottom = newbottom;
