@@ -1,26 +1,5 @@
-/*** /
- 
- This file is part of Golly, a Game of Life Simulator.
- Copyright (C) 2013 Andrew Trevorrow and Tomas Rokicki.
- 
- This program is free software; you can redistribute it and/or
- modify it under the terms of the GNU General Public License
- as published by the Free Software Foundation; either version 2
- of the License, or (at your option) any later version.
- 
- This program is distributed in the hope that it will be useful,
- but WITHOUT ANY WARRANTY; without even the implied warranty of
- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- GNU General Public License for more details.
- 
- You should have received a copy of the GNU General Public License
- along with this program; if not, write to the Free Software
- Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
- 
- Web site:  http://sourceforge.net/projects/golly
- Authors:   rokicki@gmail.com  andrew@trevorrow.com
- 
- / ***/
+// This file is part of Golly.
+// See docs/License.html for the copyright notice.
 
 #include "wx/wxprec.h"     // for compilers that support precompilation
 #ifndef WX_PRECOMP
@@ -329,6 +308,8 @@ void StatusBar::DrawStatusBar(wxDC& dc, wxRect& updaterect)
             strbuf = _("Population = ");
             if (viewptr->nopattupdate) {
                 strbuf += _("0");
+            } else if (mainptr->generating && !showpopulation) {
+                strbuf += _("disabled");
             } else {
                 bigint popcount = currlayer->algo->getPopulation();
                 if (popcount.sign() < 0) {
@@ -391,6 +372,8 @@ void StatusBar::DrawStatusBar(wxDC& dc, wxRect& updaterect)
             strbuf = _("Population=");
             if (viewptr->nopattupdate) {
                 strbuf += _("0");
+            } else if (mainptr->generating && !showpopulation) {
+                strbuf += _("disabled");
             } else {
                 bigint popcount = currlayer->algo->getPopulation();
                 if (popcount.sign() < 0) {
@@ -494,6 +477,16 @@ bool StatusBar::ClickInGenBox(int x, int y)
 
 // -----------------------------------------------------------------------------
 
+bool StatusBar::ClickInPopBox(int x, int y)
+{
+    if (showexact)
+        return x >= 0 && y > (POPLINE+DESCHT-LINEHT) && y <= (POPLINE+DESCHT);
+    else
+        return x >= h_pop && x <= h_scale - 20 && y <= (BASELINE1+DESCHT);
+}
+
+// -----------------------------------------------------------------------------
+
 bool StatusBar::ClickInScaleBox(int x, int y)
 {
     if (showexact)
@@ -516,7 +509,7 @@ bool StatusBar::ClickInStepBox(int x, int y)
 
 void StatusBar::OnMouseDown(wxMouseEvent& event)
 {
-    if (inscript) return;    // let script control scale and step
+    if (inscript) return;    // let script control scale, step, etc
     ClearMessage();
     
     if ( ClickInGenBox(event.GetX(), event.GetY()) && !mainptr->generating ) {
@@ -526,6 +519,12 @@ void StatusBar::OnMouseDown(wxMouseEvent& event)
             mainptr->SetGeneration();
         }
         
+    } else if ( ClickInPopBox(event.GetX(), event.GetY()) ) {
+        if (mainptr->generating) {
+            mainptr->ToggleShowPopulation();
+            mainptr->UpdateMenuItems();
+        }
+        
     } else if ( ClickInScaleBox(event.GetX(), event.GetY()) ) {
         if (viewptr->GetMag() != 0) {
             // reset scale to 1:1
