@@ -1,26 +1,5 @@
-/*** /
- 
- This file is part of Golly, a Game of Life Simulator.
- Copyright (C) 2013 Andrew Trevorrow and Tomas Rokicki.
- 
- This program is free software; you can redistribute it and/or
- modify it under the terms of the GNU General Public License
- as published by the Free Software Foundation; either version 2
- of the License, or (at your option) any later version.
- 
- This program is distributed in the hope that it will be useful,
- but WITHOUT ANY WARRANTY; without even the implied warranty of
- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- GNU General Public License for more details.
- 
- You should have received a copy of the GNU General Public License
- along with this program; if not, write to the Free Software
- Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
- 
- Web site:  http://sourceforge.net/projects/golly
- Authors:   rokicki@gmail.com  andrew@trevorrow.com
- 
- / ***/
+// This file is part of Golly.
+// See docs/License.html for the copyright notice.
 
 #include "wx/wxprec.h"      // for compilers that support precompilation
 #ifndef WX_PRECOMP
@@ -67,7 +46,7 @@ enum {
     DELETE_BUTT,
     NUM_BUTTONS,        // must be after all buttons
     ID_SLIDER,          // for slider
-    ID_SCROLL,          // for scroll bar
+    ID_SCROLLBAR,       // for scroll bar
     ID_AUTOTIMER        // for autotimer
 };
 
@@ -142,7 +121,7 @@ EVT_PAINT            (              TimelineBar::OnPaint)
 EVT_LEFT_DOWN        (              TimelineBar::OnMouseDown)
 EVT_BUTTON           (wxID_ANY,     TimelineBar::OnButton)
 EVT_COMMAND_SCROLL   (ID_SLIDER,    TimelineBar::OnSlider)
-EVT_COMMAND_SCROLL   (ID_SCROLL,    TimelineBar::OnScroll)
+EVT_COMMAND_SCROLL   (ID_SCROLLBAR, TimelineBar::OnScroll)
 EVT_TIMER            (ID_AUTOTIMER, TimelineBar::OnAutoTimer)
 END_EVENT_TABLE()
 
@@ -292,7 +271,7 @@ TimelineBar::TimelineBar(wxWindow* parent, wxCoord xorg, wxCoord yorg, int wd, i
 #endif
     x = xpos + 20;
     y = (TBARHT - (scrollbarht + 1)) / 2;
-    framebar = new wxScrollBar(this, ID_SCROLL, wxPoint(x, y),
+    framebar = new wxScrollBar(this, ID_SCROLLBAR, wxPoint(x, y),
                                wxSize(scrollbarwd, scrollbarht),
                                wxSB_HORIZONTAL);
     if (framebar == NULL) Fatal(_("Failed to create timeline scroll bar!"));
@@ -866,6 +845,10 @@ void UpdateTimelineBar()
         } else {
             tbarptr->Refresh(false);
         }
+        #ifdef __WXGTK__
+            // avoid bug that can cause buttons to lose their bitmaps
+            tbarptr->Update();
+        #endif
     }
 }
 
