@@ -1,26 +1,5 @@
-/*** /
- 
- This file is part of Golly, a Game of Life Simulator.
- Copyright (C) 2013 Andrew Trevorrow and Tomas Rokicki.
- 
- This program is free software; you can redistribute it and/or
- modify it under the terms of the GNU General Public License
- as published by the Free Software Foundation; either version 2
- of the License, or (at your option) any later version.
- 
- This program is distributed in the hope that it will be useful,
- but WITHOUT ANY WARRANTY; without even the implied warranty of
- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- GNU General Public License for more details.
- 
- You should have received a copy of the GNU General Public License
- along with this program; if not, write to the Free Software
- Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
- 
- Web site:  http://sourceforge.net/projects/golly
- Authors:   rokicki@gmail.com  andrew@trevorrow.com
- 
- / ***/
+// This file is part of Golly.
+// See docs/License.html for the copyright notice.
 
 #include "wx/wxprec.h"     // for compilers that support precompilation
 #ifndef WX_PRECOMP
@@ -34,7 +13,7 @@
 #include "wxgolly.h"       // for wxGetApp, viewptr, mainptr
 #include "wxview.h"        // for viewptr->...
 #include "wxmain.h"        // for mainptr->...
-#include "wxscript.h"      // for inscript, PassKeyToScript
+#include "wxscript.h"      // for inscript, showprogress, PassKeyToScript
 #include "wxprefs.h"       // for allowbeep
 #include "wxutils.h"
 
@@ -51,25 +30,43 @@
 
 // -----------------------------------------------------------------------------
 
-void Note(const wxString& msg)
+void Note(const wxString& msg, bool showCancel)
 {
     wxString title = wxGetApp().GetAppName() + _(" note:");
 #ifdef __WXMAC__
     wxSetCursor(*wxSTANDARD_CURSOR);
 #endif
-    wxMessageBox(msg, title, wxOK | wxICON_INFORMATION, wxGetActiveWindow());
+    if (inscript && showCancel) {
+        // add a Cancel button so user can abort script
+        int button = wxMessageBox(msg, title, wxOK | wxCANCEL | wxICON_INFORMATION,
+                                  wxGetActiveWindow());
+        if (button == wxCANCEL) {
+            PassKeyToScript(WXK_ESCAPE);     // abort script
+        }
+    } else {
+        wxMessageBox(msg, title, wxOK | wxICON_INFORMATION, wxGetActiveWindow());
+    }
 }
 
 // -----------------------------------------------------------------------------
 
-void Warning(const wxString& msg)
+void Warning(const wxString& msg, bool showCancel)
 {
     Beep();
     wxString title = wxGetApp().GetAppName() + _(" warning:");
 #ifdef __WXMAC__
     wxSetCursor(*wxSTANDARD_CURSOR);
 #endif
-    wxMessageBox(msg, title, wxOK | wxICON_EXCLAMATION, wxGetActiveWindow());
+    if (inscript && showCancel) {
+        // add a Cancel button so user can abort script
+        int button = wxMessageBox(msg, title, wxOK | wxCANCEL | wxICON_EXCLAMATION,
+                                  wxGetActiveWindow());
+        if (button == wxCANCEL) {
+            PassKeyToScript(WXK_ESCAPE);     // abort script
+        }
+    } else {
+        wxMessageBox(msg, title, wxOK | wxICON_EXCLAMATION, wxGetActiveWindow());
+    }
 }
 
 // -----------------------------------------------------------------------------
@@ -131,7 +128,7 @@ StringDialog::StringDialog(wxWindow* parent, const wxString& title,
     
     // position the controls
     wxBoxSizer* stdhbox = new wxBoxSizer(wxHORIZONTAL);
-    stdhbox->Add(stdbutts, 1, wxGROW | wxALIGN_CENTER_VERTICAL | wxRIGHT, STDHGAP);
+    stdhbox->Add(stdbutts, 1, wxALIGN_CENTER_VERTICAL | wxRIGHT, STDHGAP);
     wxSize minsize = stdhbox->GetMinSize();
     if (minsize.GetWidth() < 250) {
         minsize.SetWidth(250);
@@ -287,7 +284,7 @@ IntegerDialog::IntegerDialog(wxWindow* parent,
     
     // position the controls
     wxBoxSizer* stdhbox = new wxBoxSizer(wxHORIZONTAL);
-    stdhbox->Add(stdbutts, 1, wxGROW | wxALIGN_CENTER_VERTICAL | wxRIGHT, STDHGAP);
+    stdhbox->Add(stdbutts, 1, wxALIGN_CENTER_VERTICAL | wxRIGHT, STDHGAP);
     wxSize minsize = stdhbox->GetMinSize();
     if (minsize.GetWidth() < 250) {
         minsize.SetWidth(250);
@@ -424,6 +421,8 @@ void ProgressHandler::OnKeyDown(wxKeyEvent& event)
 
 void BeginProgress(const wxString& dlgtitle)
 {
+    if (inscript && !showprogress) return;
+
     if (progdlg) {
         // better do this in case of nested call
         delete progdlg;
@@ -449,6 +448,8 @@ void BeginProgress(const wxString& dlgtitle)
 
 bool AbortProgress(double fraction_done, const wxString& newmsg)
 {
+    if (inscript && !showprogress) return false;
+
     long msecs = progwatch->Time();
     if (progdlg) {
         if (msecs < prognext) return false;
@@ -494,6 +495,8 @@ bool AbortProgress(double fraction_done, const wxString& newmsg)
 
 void EndProgress()
 {
+    if (inscript && !showprogress) return;
+
     if (progdlg) {
 #if defined(__WXMAC__)
         // remove and delete ProgressHandler
